# -*- encoding: utf-8 -*-
"""
GObj :class:`HttpResponse`
=========================

Code copied from waitress.task.py adapted to ginsfsm.

.. autoclass:: HttpResponse
    :members:

This gobj can be used as mix.

"""

import time

from ginsfsm import __version__
from ginsfsm.compat import tobytes
from ginsfsm.protocols.http.common.utilities import build_http_date


class HttpResponse(object):
    def __init__(self, request):
        self.request = request
        self.channel = request.channel
        self.gsock = request.channel.gsock
        self.response_headers = []
        version = request.version
        if version not in ('1.0', '1.1'):
            # fall back to a version we support.
            version = '1.0'
        self.version = version

        # these variables were in the class section
        self.close_on_finish = False
        self.status = '200 OK'
        self.wrote_header = False
        self.start_time = 0
        self.content_length = None
        self.content_bytes_written = 0
        self.logged_write_excess = False
        self.complete = False
        self.chunked_response = False

    def execute(self):
        """ Execute the response.
            Must be overridden.
        """

    def build_response_header(self):
        version = self.version
        # Figure out whether the connection should be closed.
        connection = self.request.headers.get('CONNECTION', '').lower()
        response_headers = self.response_headers
        content_length_header = None
        date_header = None
        server_header = None
        connection_close_header = None

        for i, (headername, headerval) in enumerate(response_headers):
            headername = '-'.join(
                [x.capitalize() for x in headername.split('-')]
            )
            if headername == 'Content-Length':
                content_length_header = headerval
            if headername == 'Date':
                date_header = headerval
            if headername == 'Server':
                server_header = headerval
            if headername == 'Connection':
                connection_close_header = headerval.lower()
            # replace with properly capitalized version
            response_headers[i] = (headername, headerval)

        if content_length_header is None and self.content_length is not None:
            content_length_header = str(self.content_length)
            self.response_headers.append(
                ('Content-Length', content_length_header)
            )

        def close_on_finish():
            if connection_close_header is None:
                response_headers.append(('Connection', 'close'))
            self.close_on_finish = True

        if version == '1.0':
            if connection == 'keep-alive':
                if not content_length_header:
                    close_on_finish()
                else:
                    response_headers.append(('Connection', 'Keep-Alive'))
            else:
                close_on_finish()

        elif version == '1.1':
            if connection == 'close':
                close_on_finish()

            if not content_length_header:
                response_headers.append(('Transfer-Encoding', 'chunked'))
                self.chunked_response = True
                if not self.close_on_finish:
                    close_on_finish()

            # under HTTP 1.1 keep-alive is default, no need to set the header
        else:
            raise AssertionError('neither HTTP/1.0 or HTTP/1.1')

        # Set the Server and Date field, if not yet specified. This is needed
        # if the server is used as a proxy.
        ident = 'ginsfsm-%s' % __version__
        if not server_header:
            response_headers.append(('Server', ident))
        else:
            response_headers.append(('Via', ident))
        if not date_header:
            if not self.start_time:
                self.start_time = time.time()
            response_headers.append(('Date', build_http_date(self.start_time)))

        first_line = 'HTTP/%s %s' % (self.version, self.status)
        next_lines = ['%s: %s' % hv for hv in sorted(self.response_headers)]
        lines = [first_line] + next_lines
        res = '%s\r\n\r\n' % '\r\n'.join(lines)
        return tobytes(res)

    def remove_content_length_header(self):
        for i, (header_name, header_value) in enumerate(self.response_headers):
            if header_name.lower() == 'content-length':
                del self.response_headers[i]

    def start(self):
        self.start_time = time.time()

    def finish(self):
        """ Finishes this response,
            flushing output buffer,
            and ending the HTTP request.
        """
        if not self.wrote_header:
            self.write(b'')
        if self.chunked_response:
            # not self.write, it will chunk it!
            self.channel.send_event(
                self.gsock,
                'EV_WRITE_OUTPUT_DATA',
                data=b'0\r\n\r\n'
            )
        if self.close_on_finish:
            self.gsock.close_when_flushed = True
        self.request._close()
        self.flush()

    def flush(self, callback=None):
        """ Flush output buffer.
        TODO: posibilita que se ejecute un callback al recibir
        el evento EV_TRANSMIT_READY
        """
        self.channel.send_event(self.gsock, 'EV_FLUSH_OUTPUT_DATA')
        if callback:
            # TODO: must be executed at on event WRITE
            # instead next pooling cycle.
            self.channel.gaplic.add_callback(callback)

    def write(self, data):
        """ Write data to output buffer.
        """
        if not self.complete:
            raise RuntimeError('start_response was not called before body '
                               'written')
        gsock = self.gsock
        if not self.wrote_header:
            rh = self.build_response_header()
            self.channel.send_event(gsock, 'EV_WRITE_OUTPUT_DATA', data=rh)
            self.wrote_header = True

        if data:
            towrite = data
            cl = self.content_length
            if self.chunked_response:
                # use chunked encoding response
                towrite = tobytes(hex(len(data))[2:].upper()) + b'\r\n'
                towrite += data + b'\r\n'
            elif cl is not None:
                towrite = data[:cl - self.content_bytes_written]
                self.content_bytes_written += len(towrite)
                if towrite != data and not self.logged_write_excess:
                    self.gsock.logger and self.gsock.logger.error(
                        'application-written content exceeded the number of '
                        'bytes specified by Content-Length header (%s)' % cl)
                    self.logged_write_excess = True
            if towrite:
                self.channel.send_event(
                    gsock,
                    'EV_WRITE_OUTPUT_DATA',
                    data=towrite)


class HttpErrorResponse(HttpResponse):
    """ An error task produces an error response """

    def __init__(self, request):
        HttpResponse.__init__(self, request)
        self.complete = True

    def execute(self):
        e = self.request.error
        body = '%s\r\n\r\n%s' % (e.reason, e.body)
        tag = '\r\n\r\n(generated by ginsfsm)'
        body = body + tag
        self.status = '%s %s' % (e.code, e.reason)
        cl = len(body)
        self.content_length = cl
        self.response_headers.append(('Content-Length', str(cl)))
        self.response_headers.append(('Content-Type', 'text/plain'))
        self.response_headers.append(('Connection', 'close'))
        self.close_on_finish = True
        self.write(tobytes(body))


class HttpSimpleOkResponse(HttpResponse):
    """ An simple OK response """

    def __init__(self, request, body, toclose=True):
        HttpResponse.__init__(self, request)
        self.complete = True
        self.body = body
        self.toclose = toclose

    def execute(self):
        cl = len(self.body)
        self.content_length = cl
        self.response_headers.append(('Content-Length', str(cl)))
        self.response_headers.append(('Content-Type', 'text/plain'))
        if self.toclose:
            self.response_headers.append(('Connection', 'close'))
            self.close_on_finish = True
        self.write(tobytes(self.body))



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""HTTP Request Parser

This server uses asyncore to accept connections and do initial
processing but threads to do work.
"""
import re
from io import BytesIO
import sys

from ginsfsm import __version__
from ginsfsm.buffers import ReadOnlyFileBasedBuffer
from ginsfsm.compat import (
    tostr,
    urlparse,
    unquote_bytes_to_wsgi,
)

from ginsfsm.buffers import OverflowableBuffer

from ginsfsm.protocols.http.common.receiver import (
    FixedStreamReceiver,
    ChunkedReceiver,
)

from ginsfsm.protocols.http.common.utilities import (
    find_double_newline,
    RequestEntityTooLarge,
    RequestHeaderFieldsTooLarge,
)

rename_headers = {
    'CONTENT_LENGTH': 'CONTENT_LENGTH',
    'CONTENT_TYPE': 'CONTENT_TYPE',
}


def build_environment(request, server_name, server_port, remote_addr):
    """ Build a wsgi enviroment from a request.
        You need to know the server name/port and the remote addr
    """
    environ = {}
    path = request.path
    while path and path.startswith('/'):
        path = path[1:]

    environ['REQUEST_METHOD'] = request.command.upper()
    environ['SERVER_PORT'] = server_port
    environ['SERVER_NAME'] = server_name
    environ['SERVER_SOFTWARE'] = 'ginsfsm-%s' % __version__
    environ['SERVER_PROTOCOL'] = 'HTTP/%s' % request.version
    environ['SCRIPT_NAME'] = ''
    environ['PATH_INFO'] = '/' + path
    environ['QUERY_STRING'] = request.query
    environ['REMOTE_ADDR'] = remote_addr

    for key, value in request.headers.items():
        value = value.strip()
        mykey = rename_headers.get(key, None)
        if mykey is None:
            mykey = 'HTTP_%s' % key
        if not mykey in environ:
            environ[mykey] = value

    # the following environment variables are required by the WSGI spec
    environ['wsgi.version'] = (1, 0)
    environ['wsgi.url_scheme'] = request.url_scheme
    environ['wsgi.errors'] = sys.stderr  # apps should use the logging module
    environ['wsgi.multithread'] = False
    environ['wsgi.multiprocess'] = False
    environ['wsgi.run_once'] = False
    environ['wsgi.input'] = request.get_body_stream()
    environ['wsgi.file_wrapper'] = ReadOnlyFileBasedBuffer
    return environ


class HTTPRequestParser(object):
    """A structure that collects the HTTP request.
    """
    # Other attributes: first_line, header, headers, command, uri, version,
    # path, query, fragment

    def __init__(self, channel=None):
        # headers is a mapping containing keys translated to uppercase
        # with dashes turned into underscores.
        self.channel = None
        self.url_scheme = 'http'
        self.inbuf_overflow = 524288
        self.max_request_header_size = 262144
        self.max_request_body_size = 1073741824
        if channel:
            self.channel = channel
            self.url_scheme = channel.config.url_scheme
            self.inbuf_overflow = channel.config.inbuf_overflow
            self.max_request_header_size = \
                channel.config.max_request_header_size
            self.max_request_body_size = channel.config.max_request_body_size

        self.headers = {}

        # these variables were in the class section
        self.completed = False  # Set once request is completed.
        self.empty = False        # Set if no request was made.
        self.expect_continue = False  # client sent "Expect: 100-continue" hder
        self.headers_finished = False  # True when headers have been read
        self.header_plus = b''
        self.chunked = False
        self.content_length = 0
        self.header_bytes_received = 0
        self.body_bytes_received = 0
        self.body_rcv = None
        self.version = '1.0'
        self.error = None
        self.connection_close = False

    def received(self, data):
        """
        Receives the HTTP stream for one request.  Returns the number of
        bytes consumed.  Sets the completed flag once both the header and the
        body have been received.
        """
        if self.completed:
            return 0  # Can't consume any more.
        datalen = len(data)
        br = self.body_rcv
        if br is None:
            # In header.
            s = self.header_plus + data
            index = find_double_newline(s)
            if index >= 0:
                # Header finished.
                header_plus = s[:index]
                consumed = len(data) - (len(s) - index)
                # Remove preceeding blank lines.
                header_plus = header_plus.lstrip()
                if not header_plus:
                    self.empty = True
                    self.completed = True
                else:
                    self.parse_header(header_plus)
                    if self.body_rcv is None:
                        # no content-length header and not a t-e: chunked
                        # request
                        self.completed = True
                    if self.content_length > 0:
                        max_body = self.max_request_body_size
                        # we won't accept this request if the content-length
                        # is too large
                        if self.content_length >= max_body:
                            self.error = RequestEntityTooLarge(
                                'exceeds max_body of %s' % max_body)
                            self.completed = True
                self.headers_finished = True
                return consumed
            else:
                # Header not finished yet.
                self.header_bytes_received += datalen
                max_header = self.max_request_header_size
                if self.header_bytes_received >= max_header:
                    self.parse_header(b'GET / HTTP/1.0\n')
                    self.error = RequestHeaderFieldsTooLarge(
                        'exceeds max_header of %s' % max_header)
                    self.completed = True
                self.header_plus = s
                return datalen
        else:
            # In body.
            consumed = br.received(data)
            self.body_bytes_received += consumed
            max_body = self.max_request_body_size
            if self.body_bytes_received >= max_body:
                # this will only be raised during t-e: chunked requests
                self.error = RequestEntityTooLarge(
                    'exceeds max_body of %s' % max_body)
                self.completed = True
            elif br.error:
                # garbage in chunked encoding input probably
                self.error = br.error
                self.completed = True
            elif br.completed:
                self.completed = True
            return consumed

    def parse_header(self, header_plus):
        """
        Parses the header_plus block of text (the headers plus the
        first line of the request).
        """
        index = header_plus.find(b'\n')
        if index >= 0:
            first_line = header_plus[:index].rstrip()
            header = header_plus[index + 1:]
        else:
            first_line = header_plus.rstrip()
            header = b''

        self.first_line = first_line  # for testing

        lines = get_header_lines(header)

        headers = self.headers
        for line in lines:
            index = line.find(b':')
            if index > 0:
                key = line[:index]
                value = line[index + 1:].strip()
                key1 = tostr(key.upper().replace(b'-', b'_'))
                # If a header already exists, we append subsequent values
                # seperated by a comma. Applications already need to handle
                # the comma seperated values, as HTTP front ends might do
                # the concatenation for you (behavior specified in RFC2616).
                try:
                    headers[key1] += tostr(b', ' + value)
                except KeyError:
                    headers[key1] = tostr(value)
            # else there's garbage in the headers?

        # command, uri, version will be bytes
        command, uri, version = crack_request_first_line(first_line)
        version = tostr(version)
        command = tostr(command)
        self.command = self.method = command
        self.version = version
        (self.proxy_scheme,
         self.proxy_netloc,
         self.path,
         self.query, self.fragment) = split_uri(uri)
        self.url_scheme = self.url_scheme
        connection = headers.get('CONNECTION', '')

        if version == '1.0':
            if connection.lower() != 'keep-alive':
                self.connection_close = True

        if version == '1.1':
            te = headers.get('TRANSFER_ENCODING', '')
            if te == 'chunked':
                self.chunked = True
                buf = OverflowableBuffer(self.inbuf_overflow)
                self.body_rcv = ChunkedReceiver(buf)
            expect = headers.get('EXPECT', '').lower()
            self.expect_continue = expect == '100-continue'
            if connection.lower() == 'close':
                self.connection_close = True

        if not self.chunked:
            try:
                cl = int(headers.get('CONTENT_LENGTH', 0))
            except ValueError:
                cl = 0
            self.content_length = cl
            if cl > 0:
                buf = OverflowableBuffer(self.inbuf_overflow)
                self.body_rcv = FixedStreamReceiver(cl, buf)

    def get_body_stream(self):
        body_rcv = self.body_rcv
        if body_rcv is not None:
            return body_rcv.getfile()
        else:
            return BytesIO()

    def _close(self):
        body_rcv = self.body_rcv
        if body_rcv is not None:
            body_rcv.getbuf()._close()


class HTTPResponseParser(object):
    """A structure that collects the HTTP response.
    """
    # Other attributes: first_line, header, headers, command, uri, version,
    # path, query, fragment

    def __init__(self, channel=None):
        # headers is a mapping containing keys translated to uppercase
        # with dashes turned into underscores.
        self.channel = None
        self.url_scheme = 'http'
        self.inbuf_overflow = 524288
        self.max_request_header_size = 262144
        self.max_request_body_size = 1073741824
        if channel:
            self.channel = channel
            self.url_scheme = channel.config.url_scheme
            self.inbuf_overflow = channel.config.inbuf_overflow
            self.max_request_header_size = \
                channel.config.max_request_header_size
            self.max_request_body_size = channel.config.max_request_body_size

        self.headers = {}

        # these variables were in the class section
        self.completed = False  # Set once request is completed.
        self.empty = False        # Set if no request was made.
        self.expect_continue = False  # client sent "Expect: 100-continue" hder
        self.headers_finished = False  # True when headers have been read
        self.header_plus = b''
        self.chunked = False
        self.content_length = 0
        self.header_bytes_received = 0
        self.body_bytes_received = 0
        self.body_rcv = None
        self.version = '1.0'
        self.error = None
        self.connection_close = False

    def received(self, data):
        """
        Receives the HTTP stream for one request.  Returns the number of
        bytes consumed.  Sets the completed flag once both the header and the
        body have been received.
        """
        if self.completed:
            return 0  # Can't consume any more.
        datalen = len(data)
        br = self.body_rcv
        if br is None:
            # In header.
            s = self.header_plus + data
            index = find_double_newline(s)
            if index >= 0:
                # Header finished.
                header_plus = s[:index]
                consumed = len(data) - (len(s) - index)
                # Remove preceeding blank lines.
                header_plus = header_plus.lstrip()
                if not header_plus:
                    self.empty = True
                    self.completed = True
                else:
                    self.parse_header(header_plus)
                    if self.body_rcv is None:
                        # no content-length header and not a t-e: chunked
                        # request
                        self.completed = True
                    if self.content_length > 0:
                        max_body = self.max_request_body_size
                        # we won't accept this request if the content-length
                        # is too large
                        if self.content_length >= max_body:
                            self.error = RequestEntityTooLarge(
                                'exceeds max_body of %s' % max_body)
                            self.completed = True
                self.headers_finished = True
                return consumed
            else:
                # Header not finished yet.
                self.header_bytes_received += datalen
                max_header = self.max_request_header_size
                if self.header_bytes_received >= max_header:
                    self.parse_header(b'GET / HTTP/1.0\n')
                    self.error = RequestHeaderFieldsTooLarge(
                        'exceeds max_header of %s' % max_header)
                    self.completed = True
                self.header_plus = s
                return datalen
        else:
            # In body.
            consumed = br.received(data)
            self.body_bytes_received += consumed
            max_body = self.max_request_body_size
            if self.body_bytes_received >= max_body:
                # this will only be raised during t-e: chunked requests
                self.error = RequestEntityTooLarge(
                    'exceeds max_body of %s' % max_body)
                self.completed = True
            elif br.error:
                # garbage in chunked encoding input probably
                self.error = br.error
                self.completed = True
            elif br.completed:
                self.completed = True
            return consumed

    def parse_header(self, header_plus):
        """
        Parses the header_plus block of text (the headers plus the
        first line of the request).
        """
        index = header_plus.find(b'\n')
        if index >= 0:
            first_line = header_plus[:index].rstrip()
            header = header_plus[index + 1:]
        else:
            first_line = header_plus.rstrip()
            header = b''

        self.first_line = first_line  # for testing

        lines = get_header_lines(header)

        headers = self.headers
        for line in lines:
            index = line.find(b':')
            if index > 0:
                key = line[:index]
                value = line[index + 1:].strip()
                key1 = tostr(key.upper().replace(b'-', b'_'))
                # If a header already exists, we append subsequent values
                # seperated by a comma. Applications already need to handle
                # the comma seperated values, as HTTP front ends might do
                # the concatenation for you (behavior specified in RFC2616).
                try:
                    headers[key1] += tostr(b', ' + value)
                except KeyError:
                    headers[key1] = tostr(value)
            # else there's garbage in the headers?

        # version, status, status_info will be bytes
        version, status, status_info = crack_response_first_line(first_line)
        try:
            status = int(tostr(status))
        except ValueError:
            status = 0

        version = tostr(version)
        status_info = tostr(status_info)
        self.version = version
        self.status = status
        self.status_info = status_info
        self.url_scheme = self.url_scheme
        connection = headers.get('CONNECTION', '')

        if version == '1.0':
            if connection.lower() != 'keep-alive':
                self.connection_close = True

        if version == '1.1':
            te = headers.get('TRANSFER_ENCODING', '')
            if te == 'chunked':
                self.chunked = True
                buf = OverflowableBuffer(self.inbuf_overflow)
                self.body_rcv = ChunkedReceiver(buf)
            expect = headers.get('EXPECT', '').lower()
            self.expect_continue = expect == '100-continue'
            if connection.lower() == 'close':
                self.connection_close = True

        if not self.chunked:
            try:
                cl = int(headers.get('CONTENT_LENGTH', 0))
            except ValueError:
                cl = 0
            self.content_length = cl
            if cl > 0:
                buf = OverflowableBuffer(self.inbuf_overflow)
                self.body_rcv = FixedStreamReceiver(cl, buf)

    def get_body_stream(self):
        body_rcv = self.body_rcv
        if body_rcv is not None:
            return body_rcv.getfile()
        else:
            return BytesIO()

    def _close(self):
        body_rcv = self.body_rcv
        if body_rcv is not None:
            body_rcv.getbuf()._close()


def split_uri(uri):
    # urlsplit handles byte input by returning bytes on py3, so
    # scheme, netloc, path, query, and fragment are bytes
    scheme, netloc, path, query, fragment = urlparse.urlsplit(uri)
    return (
        tostr(scheme),
        tostr(netloc),
        unquote_bytes_to_wsgi(path),
        tostr(query),
        tostr(fragment),
    )


def get_header_lines(header):
    """
    Splits the header into lines, putting multi-line headers together.
    """
    r = []
    lines = header.split(b'\n')
    for line in lines:
        if line.startswith((b' ', b'\t')):
            r[-1] = r[-1] + line[1:]
        else:
            r.append(line)
    return r


request_first_line_re = re.compile(
b'([^ ]+) ((?:[^ :?#]+://[^ ?#/]*(?:[0-9]{1,5})?)?[^ ]+)(( HTTP/([0-9.]+))$|$)')


def crack_request_first_line(line):
    m = request_first_line_re.match(line)
    if m is not None and m.end() == len(line):
        if m.group(3):
            version = m.group(5)
        else:
            version = None
        command = m.group(1).upper()
        uri = m.group(2)
        return command, uri, version
    else:
        return b'', b'', b''


response_first_line_re = re.compile(b'HTTP/(1.[01]) ([0-9]+) (.*)')


def crack_response_first_line(line):
    m = response_first_line_re.match(line)

    if m is not None and m.end() == len(line):
        version = m.group(1)
        status = m.group(2)
        status_info = m.group(3)
        return version, status, status_info
    else:
        return b'', b'', b''



""" Inter event example.

.. image:: inter_event_schema.png


See the examples in directories:

* ginsfsm.examples.inter_event.ap1_banco
* ginsfsm.examples.inter_event.ap2_financiera
* ginsfsm.examples.inter_event.ap3_bolsa
* ginsfsm.examples.inter_event.ap4_publicidad

"""



import logging
logging.basicConfig(level=logging.DEBUG)

from ginsfsm.gobj import GObj
from ginsfsm.c_connex import GConnex
from ginsfsm.gaplic import GAplic, setup_gaplic_thread
from ginsfsm.c_timer import GTimer
from ginsfsm.c_srv_sock import GServerSock


#===============================================================
#   Server: emit a message to clients each 5 seconds
#===============================================================
def ac_clisrv_timeout(self, event):
    self.set_timeout(5)
    for child in self.clients:
        self.send_event(child, 'EV_SEND_DATA', data='xxxxxx')


def ac_clisrv_connected(self, event):
    self.clients.add(event.gsock)


def ac_clisrv_disconnected(self, event):
    self.clients.remove(event.gsock)
    self.destroy_gobj(event.source[-1])


def ac_clisrv_rx_data(self, event):
    # Do echo
    self.send_event(event.source[-1], 'EV_SEND_DATA', data=event.data)


SERVER_FSM = {
    'event_list': (
        'EV_SET_TIMER: bottom output',
        'EV_TIMEOUT: bottom input',
        'EV_CONNECTED: bottom input',
        'EV_DISCONNECTED: bottom input',
        'EV_RX_DATA:bottom input',
    ),
    'state_list': ('ST_IDLE',),
    'machine': {
        'ST_IDLE':
        (
            ('EV_TIMEOUT',          ac_clisrv_timeout,         'ST_IDLE'),
            ('EV_CONNECTED',        ac_clisrv_connected,       'ST_IDLE'),
            ('EV_DISCONNECTED',     ac_clisrv_disconnected,    'ST_IDLE'),
            ('EV_RX_DATA',          ac_clisrv_rx_data,         'ST_IDLE'),
        ),
    }
}


SERVER_GCONFIG = {  # type, default_value, flag, validate_function, desc
    'verbose': [int, 1, 0, None, "Increase output verbosity. Values [0,1,2]"],
}


class OnServer(GObj):
    """  Server GObj.

    .. ginsfsm::
       :fsm: SERVER_FSM
       :gconfig: SERVER_GCONFIG

    *Input-Events:*

        * :attr:`'EV_TIMEOUT'`: Timer over.
            Start the machine.

        * :attr:`'EV_CONNECTED'`: New client.

        * :attr:`'EV_DISCONNECTED'`: Client disconnected.

        * :attr:`'EV_RX_DATA'`: Receiving data.

    *Output-Events:*

    """
    def __init__(self):
        GObj.__init__(self, SERVER_FSM, SERVER_GCONFIG)
        self.clients = set()

    def start_up(self):
        self.timer = self.create_gobj(
            None,
            GTimer,
            self)
        self.set_timeout(5)

        self.server = self.create_gobj(
            None,
            GServerSock,
            self,
            host='0.0.0.0',
            port=9999,
            transmit_ready_event_name=None,
        )

    def set_timeout(self, seconds):
        self.send_event(self.timer, 'EV_SET_TIMER', seconds=seconds)

    def clear_timeout(self):
        self.send_event(self.timer, 'EV_SET_TIMER', seconds=-1)


#===============================================================
#                   Driver
#===============================================================
def ac_rx_data(self, event):
    data = event.kw['data']
    print("RECIBO======> %s" % data)
    self.broadcast_event('EV_UPDATE_DRIVER', data=data)


def ac_comando(self, event):
    self.broadcast_event('EV_UPDATE_DRIVER', data='QUE SIIIIIIIIII')


DRIVER1_FSM = {
    'event_list': (
        'EV_UPDATE_DRIVER:top output',
        'EV_RX_DATA:bottom input',
        'EV_COMANDO: top input',
    ),
    'state_list': ('ST_IDLE',),
    'machine': {
        'ST_IDLE':
        (
            ('EV_RX_DATA',      ac_rx_data,     'ST_IDLE'),
            ('EV_COMANDO',      ac_comando,     'ST_IDLE'),
        ),
    }
}


DRIVER1_GCONFIG = {  # type, default_value, flag, validate_function, desc
    'host': [str, 'localhost', 0, None, ""],
    'port': [int, 10000, 0, None, ""],
}


class Driver1(GObj):
    """  Driver1 GObj.

    .. ginsfsm::
       :fsm: DRIVER1_FSM
       :gconfig: DRIVER1_GCONFIG

    *Input-Events:*

        * :attr:`'EV_RX_DATA'`: Receiving data.
            Receiving data from urls.

    *Output-Events:*

    """
    def __init__(self):
        GObj.__init__(self, DRIVER1_FSM, DRIVER1_GCONFIG)

    def start_up(self):
        """ Initialization zone."""
        self.timer = self.create_gobj(
            None,
            GTimer,
            self,
        )
        self.connex = self.create_gobj(
            'connex',
            GConnex,
            self,
            destinations=[
                (self.config.host, self.config.port),
            ],
            # only want receive EV_RX_DATA event
            connected_event_name=None,
            disconnected_event_name=None,
            transmit_ready_event_name=None,
        )


#===============================================================
#                   Main
#===============================================================
def mycallback(event):
    print("CALLBACK recibiendo %s" % event.data)

if __name__ == "__main__":
    local_conf = {
        'GObj.trace_mach': True,
        'GObj.logger': logging,
    }

    #---------------------------------------------#
    #
    #---------------------------------------------#
    ga_srv = GAplic(name='server', roles='', **local_conf)
    ga_srv.create_gobj(
        'server',
        OnServer,
        ga_srv,
    )
    thread_server = setup_gaplic_thread(ga_srv)
    thread_server.start()

    #---------------------------------------------#
    #
    #---------------------------------------------#
    local_conf = {
        'router_enabled':  True,
        'GRouter.server':  True,
        'GRouter.localhost_route_ports':  8002,
        'GRouter.trace_router':  True,
        'GObj.trace_mach':  True,
        'GObj.trace_creation':  False,
        'GObj.trace_traverse':  True,
        'GObj.trace_subscription':  False,
        'GSock.trace_dump':  True,
        'GObj.logger': logging,
    }
    ga_driver = GAplic(name='Protocolo1', roles='domotica', **local_conf)
    ga_driver.create_gobj(
        'driver',
        Driver1,
        ga_driver,
        __unique_name__=True,
        host='localhost',
        port=9999
    )
    thread_driver = setup_gaplic_thread(ga_driver)
    thread_driver.start()

    #---------------------------------------------#
    #   Code to insert in a non-gobj code.
    #       Subscribe as callback
    #   WARNING: the callback will be called from another thread of yours.
    #   PROTECT your resources!
    #---------------------------------------------#
    from ginsfsm.globals import global_get_gobj
    from ginsfsm.deferred import Deferred

    defer = Deferred(0, mycallback)
    gobj = global_get_gobj('driver', 'Protocolo1')
    if gobj:
        gobj.subscribe_event('EV_UPDATE_DRIVER', defer)

    #---------------------------------------------#
    #       Loop
    #---------------------------------------------#
    try:
        while True:
            pass
    except (KeyboardInterrupt, SystemExit):
        print('Program stopped')

    thread_server.stop()
    thread_driver.stop()
    thread_server.join()
    thread_driver.join()



# -*- coding: utf-8 -*-
""" FIFO queue implemented with a CircularFIFO buffer.
    It's not thread-safe.
"""
DEFAULT_CIRCULAR_SIZE = 8192


class CircularFullBufferError(Exception):
    """ Raised when CircularFIFO have problems."""


class CircularFIFO(object):
    def __init__(self, size=DEFAULT_CIRCULAR_SIZE):
        """ CircularFIFO buffer.
        """
        self.size = size
        self.start = 0
        self.count = 0
        self.data = bytearray(size)

    def __str__(self):
        return "size %d, count %d, start %d, data %s" % (
            self.size,
            self.count,
            self.start,
            self.data,
        )

    def putdata(self, bf, ln=0):
        """ WRITING: add `ln` bytes of `bf` to cicular fifo.
            If `ln` is 0 then put all data.
            If `ln` is greater than data len, then `ln` is limited to data len.
            Return number of written bytes.
        """
        if ln <= 0:
            ln = len(bf)
        if ln > len(bf):
            ln = len(bf)

        if ln == 0:
            return 0

        data = bf[0:ln]

        writted = ln = len(data)
        if ln > self.free_space:
            raise CircularFullBufferError(
                "ERROR full buffer, not space for %d bytes" % (ln,))

        tail = (self.start + self.count) % self.size  # write pointer
        right_len = self.size - tail  # right zone
        self.count += ln
        if ln <= right_len:
            # enough with the right zone
            self.data[tail:tail + ln] = data[:]
        else:
            # put maximum at the right zone
            self.data[tail:tail + right_len] = data[:right_len]
            ln -= right_len

            # put the rest at the left zone
            self.data[0:ln] = data[right_len:]

        return writted

    def getdata(self, ln=0):
        """ READING : Pull 'ln' bytes.
            Return the pulled data.
            If ln is <= 0 then all data is pulled.
            If there is no enough 'ln' bytes, it return all the remaining.
        """
        if ln <= 0 or ln > self.count:
            ln = self.count
        right_len = self.size - self.start  # right zone (from start to end)
        if ln <= right_len:
            # enough with the right zone
            data = self.data[self.start: self.start + ln]
            self.count -= ln
            self.start += ln
        else:
            # get all the right zone
            data = self.data[self.start: self.start + self.size]
            self.count -= ln  # decrement now!
            ln -= self.size - self.start

            # add the rest of the left zone
            data += self.data[0:ln]
            self.start = ln

        return data

    @property
    def busy_space(self):
        """READING: Return number of bytes pending of reading
        """
        return self.count

    @property
    def free_space(self):
        """ WRITTING: Return number of free byte space to write.
        """
        free_bytes = self.size - self.count
        return free_bytes



# -*- encoding: utf-8 -*-
"""
GAplic
======

Container of gobjs.

:class:`GAplic` supplies the main loop in a thread o subprocess context.

:class:`GAplic` has the next methods:

* :meth:`GAplic.create_gobj`
* :meth:`GAplic.destroy_gobj`
* :meth:`GAplic.find_unique_gobj`
* :meth:`GAplic.send_event_outside`
* :meth:`GAplic.subscribe_event_outside`
* :meth:`GAplic.unsubscribe_event_outside`
* :meth:`GAplic.add_callback`
* :meth:`GAplic.start`
* :meth:`GAplic.stop`
* :meth:`GAplic.mt_subprocess`


A gaplic can run in a thread or subprocess context,
always with the same interface.

Auxiliary functions:
====================

To run a :class:`GAplic` instance like a **thread**:
:func:`setup_gaplic_thread`.

To run a :class:`GAplic` instance like a **subprocess**:
:func:`setup_gaplic_process`.

To use configuration file .ini
with `PasteDeploy <http://pythonpaste.org/deploy>`_ in composite mode:
:func:`gaplic_factory`.


GAplic class
============

.. autoclass:: GAplic
    :members: create_gobj, destroy_gobj,
        find_unique_gobj,
        send_event_outside,
        subscribe_event_outside,
        unsubscribe_event_outside,
        add_callback,
        start,
        stop,
        mt_subprocess


Thead context
=============

Running as thread::

    ga = GAplic()
    worker = setup_gaplic_thread(ga)
    worker.start()


.. autofunction:: setup_gaplic_thread

.. autoclass:: GAplicThreadWorker
    :members: run, join

Subprocess context
==================

Running as subprocess::

    ga = GAplic()
    worker = setup_gaplic_process(ga)
    worker.start()


.. autofunction:: setup_gaplic_process

.. autoclass:: GAplicProcessWorker
    :members: run, join


Runnig several threads or subprocesses::

    from ginsfsm.gaplic import GAplic, setup_gaplic_thread

    # run one gaplic as thread
    ga_srv = GAplic(name='Server', roles='')
    srv_worker = setup_gaplic_thread(ga_srv)
    srv_worker.start()

    ga_cli = GAplic(name='Client', roles='')

    try:
        # run the main gaplic as main process
        ga_cli.start()

    except (KeyboardInterrupt, SystemExit):
        # stop the main gaplic
        ga_srv.stop()

        # wait to finish the other gaplic
        srv_worker.join()

        print('Program stopped')



Ini file configuration
======================

You can configure and run your gaplic applications with PasteDeploy.

Available are the :term:`gcreate` and :term:`gserve` commands,
similar to pcreate and pserve of Pyramid.


.. autofunction:: gaplic_factory

"""
import time
import threading
from collections import deque

from ginsfsm.globals import (
    set_global_main_gaplic,
    set_global_app,
    add_global_thread,
    add_global_subprocess,
    get_gaplic_by_thread_ident,
)
from ginsfsm.compat import (
    string_types,
    iterkeys_,
)
from ginsfsm.deferred import (
    DeferredList,
    Deferred,
)
from ginsfsm.c_sock import (
    poll_loop,
    close_all_sockets,
    _poll,
    GSock,
)
from ginsfsm.gobj import GObj


def _start_timer(seconds):
    """ Start a timer of :param:`seconds` seconds.
    The returned value must be used to check the end of the timer
    with _elapsed_timer() function.
    """
    timer = time.time()
    timer = timer + seconds
    return timer


def _elapsed_timer(value):
    """ Check if timer :param:`value` has ended.
    Return True if the timer has elapsed, False otherwise.
    WARNING: it will fail when system clock has been changed.
    TODO: check if system clock has been changed.
    """
    timer_actual = time.time()
    if timer_actual >= value:
        return True
    else:
        return False


class _XTimer(object):
    """  Group attributes for timing.
    :param:`got_timer` callback will be executed :param:`sec` seconds.
    If :param:`autostart` is True, the timer will be cyclic.
    """
    def __init__(self, sec, timer_callback, autostart):
        self.sec = sec
        self.timer_callback = timer_callback
        self.autostart = autostart
        self.next_run = 0


class Callback(object):
    """Custom implementation of the Tornado.Callback with support
    of callback timeout delays.
    """
    def __init__(self, callback, callback_time, gaplic):
        """Constructor.

        `callback`
            Callback function
        `callback_time`
            Callback timeout value (in seconds)
        `gaplic`
            gaplic (io_loop) instance
        """
        self.callback = callback
        self.callback_time = callback_time
        self.gaplic = gaplic
        self._running = False
        self.next_run = None

    def start(self, timeout=None):
        """Start callbacks"""
        self._running = True

        if timeout is None:
            timeout = self.callback_time

        self.gaplic.add_timeout(timeout, self._run)

    def stop(self):
        """Stop callbacks"""
        self._running = False

    def delay(self):
        """Delay callback"""
        self.next_run = _start_timer(self.callback_time)

    def _run(self):
        if not self._running:
            return
        # Support for shifting callback window
        if self.next_run and not _elapsed_timer(self.next_run):
            self.start(self.next_run)
            self.next_run = None
            return

        next_call = None
        try:
            next_call = self.callback()
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.gaplic.logger and self.gaplic.logger.exception(
                "ERROR in periodic callback")

        if self._running:
            self.start(next_call)


GAPLIC_FSM = {}

#def ac_deferred_callback(self, event):
#    deferred_ref = event.deferred_ref
#    self.deferred_list(deferred_ref, ext_event=event)
#    self.deferred_list.delete(deferred_ref)
#
#GAPLIC_FSM = {
#    'event_list': ('EV_DEFERRED_CALLBACK: top input',),
#    'state_list': ('ST_IDLE',),
#    'machine': {
#        'ST_IDLE':
#        (
#            ('EV_DEFERRED_CALLBACK', ac_deferred_callback, 'ST_IDLE'),
#        ),
#    }
#}

GAPLIC_GCONFIG = {
    'roles': [tuple, (), 0, None, 'Roles of gaplic'],
    'ini_settings': [
        dict, {}, 0, None,
        'The ini settings will be set to all new created gobj'
        ' by overwrite_parameters() function'
    ],
    # trace_mach is inherited from SMachine.
    'trace_mach': [bool, False, 0, None, 'Display simple machine activity'],
    # logger is inherited from SMachine.
    'logger': [None, None, 0, None, ''],
    'router_enabled': [
        bool, False, 0, None,
        'True if a (NOT Pyramid) router is enabled.'
    ],
    'all_unique_names': [
        bool, False, 0, None,
        'All named-gobjs are unique-named gobjs'
    ],
}


class GAplic(GObj):
    """ Container of gobj's running under the same process or thread.

    :param name: name of the gaplic, default is ``None``.
    :param ini_settings: keyword arguments,
        with the parameters from a ini configfile.
        The ini settings will be set to all new created gobj
        by :func:`ginsfsm.gobj.GObj.overwrite_parameters` function.


        .. note::

           The parameters can be dot named to include
           the :term:`named-gobj`'s destination of the parameters.


    GAplic is the main boss.
    Manage the timer's, event queues, etc.
    Supplies register, deregister and search or named-events.

    .. ginsfsm::
       :fsm: GAPLIC_FSM
       :gconfig: GAPLIC_GCONFIG

    Example::

        from ginsfsm.gaplic import GAplic

        if __name__ == "__main__":
            ga = GAplic(name='Example1', roles='')
            ga.create_gobj('test_aplic', GPrincipal, None)
            try:
                ga.start()
            except KeyboardInterrupt:
                print('Program stopped')

    """
    def __init__(self, name=None, roles=None, **ini_settings):
        GObj.__init__(self, GAPLIC_FSM, GAPLIC_GCONFIG)
        self.name = name
        # TODO: register in gaplic-dns: (gaplic_name, roles, urls)
        if isinstance(roles, (list, tuple)):
            self.roles = roles
        else:
            self.roles = (roles,)
        self.ini_settings = ini_settings.copy()
        # Call stop() to stop gaplic
        self.do_exit = multiprocessing.Event()
        """ threading.Event() or multiprocessing.Event() object
        to signal the stop of gaplic."""
        self.loop_timeout = 0.5     # timeout to select(),poll() function.
        """ Loop timeout. Default 0.5 seconds.
            It's the minimun timer resolution you can have.
        """
        self._impl_poll = _poll()   # Used by gsock. epoll() implementation.
        self._socket_map = {}       # Used by gsock. Dict {fd:Gobj}
        self._gotter_timers = {}    # Dict with timers  {_XTimer:timer value}
        self._qevent = deque()      # queue for post events.
        self._callbacks = []        # callbacks compatible with tornado.io_loop
        self._inside = 0            # to tab machine trace.
        self._unique_named_gobjs = {}
        self._thread_ident = None
        self._thread_name = 0
        self.gaplic = self
        self.deferred_list = DeferredList()

        logger = ini_settings.get('logger', None)
        if not logger:
            import logging
            self.logger = logging.getLogger(__name__)
        else:
            if isinstance(logger, string_types):
                import logging
                self.logger = logging.getLogger(logger)
            else:
                self.logger = logger
        if not self.logger:
            logging.basicConfig(level=logging.DEBUG)
            self.logger = logging

        self.start_up()

    def __str__(self):
        return self.name

    def __repr__(self):
        return "%r %r" % (self.name, self.roles)

    def start_up(self):
        """ Initialization zone.
        """
        self.router_enabled = self.ini_settings.pop('router_enabled', False)
        self.all_unique_names = self.ini_settings.pop('all_unique_names',
                                                      False)

        if self.router_enabled:
            self.start_up_router()
        self.logger and self.logger.info('GAplic %r initiated' % self)

    def start_up_router(self, pyramid_router_url=None, pyramid_root=None):
        from ginsfsm.router import GRouter
        self.router = self.create_gobj(
            'router',
            GRouter,
            self,
            __unique_name__=True,
            pyramid_root=pyramid_root,
            pyramid_router_url=pyramid_router_url
        )

    def _increase_inside(self):
        self._inside += 1

    def _decrease_inside(self):
        self._inside -= 1

    def _tab(self):
        if self._inside <= 0:
            spaces = 1
        else:
            spaces = self._inside * 2
        pad = ' ' * spaces
        return pad

    def create_gobj(self, name, gclass, parent, **kw):
        """ Factory function to create gobj's instances.

        Subclass of :meth:`ginsfsm.gobj.GObj.create_gobj` to do something else,
        like to let :term:`unique-named-gobj` instances.

        :param name: Name of the gobj.
            If the key `__unique_name__` passed in *kw* is True,
            then the gobj will be a :term:`unique-named-gobj` and.
            the :meth:`GObj._register_unique_gobj` will be called.
            If :meth:`GObj._register_unique_gobj` fails, a
            :exc:GObjError exception will be raised.

        :param gclass: `gclass` is the GObj type used to create the new gobj.
            It's must be a derived class of :class:`ginsfsm.gobj.GObj`.

        :param parent: parent of the new :term:`gobj`.
            If `None`, the gobj will be a :term:`principal` gobj.

        :param kw: Attributes that are added to the new :term:`gobj`.
            All the keyword arguments used in the creation function
            **are added as attributes** to ``config`` object.

            You must consult the attributes supported
            by each `gclass` type.
            The attributes must be defined in the gclass GCONFIG,
            otherwise they are ignored.

            Special kw:

                ``'__unique_name__'``: Register the gobj as unique name gobj.

        :rtype: new gobj instance.

        When a :term:`gobj` is created by the factory function, it's added to
        their parent child list :attr:`ginsfsm.gobj.GObj.dl_childs`,
        and several attributes are created:

        * **parent**: the parent :term:`gobj` of the created :term:`gobj`.
        * **gaplic**: the :term:`gaplic` of the created :term:`gobj`.

        If the `gclass` is subclass of :class:`ginsfsm.c_sock.GSock`
        two private attributes are added to the created  :term:`gobj`:

        * **_socket_map**: dictionary of open sockets.
        * **_impl_poll**: poll implementation: can be epoll, select, KQueue..
          the best found option.

        It's the base of the asynchronous behavior.
        """
        attrs = {
            'logger': self.logger,
            'gaplic': self,
            'create_gobj': self.create_gobj,
            'destroy_gobj': self.destroy_gobj,
            '_increase_inside': self._increase_inside,
            '_decrease_inside': self._decrease_inside,
            '_tab': self._tab,
        }
        if issubclass(gclass, GSock):
            attrs.update({
                '_socket_map': self._socket_map,
                '_impl_poll': self._impl_poll
            })

        if self.all_unique_names:
            attrs.update({'__unique_name__': True})

        kw.update(attrs)

        gobj = GObj.create_gobj(self, name, gclass, parent, **kw)

        return gobj

    @staticmethod
    def destroy_gobj(gobj):
        """ Destroy a gobj
        """
        if gobj.__unique_name__:
            gobj.gaplic._deregister_unique_gobj(gobj)
        GObj.destroy_gobj(gobj)

    def get_unique_named_gobjs(self):
        """ Return the list of :term:`unique-named-gobj`'s.
        """
        return [name for name in iterkeys_(self._unique_named_gobjs)]

    def _register_unique_gobj(self, gobj):
        """ Register a :term:`unique-named-gobj`.
        """
        named_gobj = self._unique_named_gobjs.get(gobj.name, None)
        if named_gobj is not None:
            self.logger and self.logger.info(
                'ERROR _register_unique_gobj() "%s" ALREADY REGISTERED' %
                gobj.name)
            return False
        self._unique_named_gobjs[gobj.name] = gobj
        self.__unique_name__ = True
        return True

    def _deregister_unique_gobj(self, gobj):
        """ Deregister a :term:`unique-named-gobj`.
        """
        named_gobj = self._unique_named_gobjs.get(gobj.name, None)
        if named_gobj is not None:
            del self._unique_named_gobjs[gobj.name]
            return True
        return False

    def find_unique_gobj(self, gobj_name):
        """ Find a :term:`unique-named-gobj`.
        """
        named_gobj = self._unique_named_gobjs.get(gobj_name, None)
        return named_gobj

    def send_event_outside(
            self,
            gaplic_name,
            role,
            gobj_name,
            event_name,
            subscriber_gobj,
            origin_role=None,
            **kw):
        """ Send an event to an external gaplic.

        :param gaplic: name of external gaplic.
        :param role: name of external role.
        :param gobj_name: name of external gobj.
        :param event_name: name of the event to send.
        :param subscriber_gobj: subscriber obj that wants receive the response.
        :param origin_role: specify role if you have two o more roles.
        :param kw: keyword arguments.

        Possible values for **kw** arguments:
            * `__subscribe_response__`: ``Bool`` Subscribe the
              response of external executed action.
              Received with the same event.
        """
        if not isinstance(gaplic_name, string_types):
            raise TypeError(
                'Destination gaplic %r must be a string'
                % (gaplic_name)
            )
        if not isinstance(role, string_types):
            raise TypeError(
                'Destination role %r must be a string'
                % (role)
            )
        if not isinstance(gobj_name, string_types):
            raise TypeError(
                'Destination gobj %r must be a string'
                % (gobj_name)
            )

        if not isinstance(event_name, string_types):
            raise TypeError(
                'Event name %r must be a string'
                % (event_name)
            )

        if not isinstance(subscriber_gobj, (string_types, GObj)):
            raise TypeError(
                'Subscriber gobj %r must be a string'
                % (subscriber_gobj)
            )

        if role in self.roles or gaplic_name == self.name:
            raise TypeError(
                "Please don't use external methods to send self."
            )

        if origin_role and not origin_role in self.roles:
            raise TypeError(
                "Unknown origin role: %s" % origin_role
            )

        if isinstance(subscriber_gobj, (GObj)):
            subscriber_gobj = subscriber_gobj.name

        subs_gobj = self.find_unique_gobj(subscriber_gobj)
        if not subs_gobj:
            raise TypeError(
                'Subscriber gobj %r must be a __unique_named__ gobj'
                % (subscriber_gobj)
            )

        return self.send_event(
            self.router,
            'EV_SEND_EVENT_OUTSIDE',
            gaplic_name=gaplic_name,
            role=role,
            gobj_name=gobj_name,
            event_name_=event_name,
            subscriber_gobj=subscriber_gobj,
            origin_role=origin_role,
            kw=kw
        )

        #return self.router.mt_send_event_to_external_gaplic(
        #    gaplic_name,
        #    role,
        #    gobj_name,
        #    event_name,
        #    subscriber_gobj,
        #    kw
        #)

    def subscribe_event_outside(
            self,
            gaplic_name,
            role,
            gobj_name,
            event_name,
            subscriber_gobj,
            origin_role=None,
            **kw):
        """ Subscribe an event of an external gaplic by name.
        """
        kw.update({'__subscribe_event__': True})
        return self.send_event_outside(
            gaplic_name,
            role,
            gobj_name,
            event_name,
            subscriber_gobj,
            origin_role=origin_role,
            **kw
        )

    def unsubscribe_event_outside(
            self, gaplic_name, role, gobj_name, event_name, subscriber_gobj, **kw):
        """ Subscribe an event of an external gaplic by name.
        """
        kw.update({'__unsubscribe_event__': True})
        return self.send_event_outside(
            gaplic_name,
            role,
            gobj_name,
            event_name,
            subscriber_gobj,
            **kw
        )

    def delete_all_references(self, gobj):
        """ Delete all references of gobj in timer and event queues.
        """
        # TODO: by the moment, be care with your event generation

    def _loop(self):
        """ process event queue, timer queue, and epoll.
        Return True if there is some remain event for be proccessed.
        Useful for testing purposes.
        """
        if not self._thread_ident:
            self._thread_ident = threading.current_thread().ident
            self._thread_name = threading.current_thread().name

        timeout = self.loop_timeout  # iniatially wait loop_timeout seconds
        remain = False  # some work pending.

        # Callbacks compatible with tornado.io_loop
        # Prevent IO event starvation by delaying new callbacks
        # to the next iteration of the event loop.
        callbacks = self._callbacks
        if callbacks:
            remain = True
            self._callbacks = []
            for callback in callbacks:
                self._run_callback(callback)

        if self._callbacks:
            # If any callbacks or timeouts called add_callback,
            # we don't want to wait in poll() before we run them.
            timeout = 0.0

        remain |= self._process_qevent()
        if remain:
            # They are remain events,
            # we don't want to wait in poll() before we run them.
            # wait some time, t
            # o avoid recursive send events that puts 100% cpu.
            # TODO: try set 0.0.
            timeout = 0.1

        poll_loop(self._socket_map, self._impl_poll, timeout)

        #
        remain |= self._process_timer()  # don't remove or the tests will fail.

        # oportunity for subclass.
        self.mt_subprocess()

        return remain

    def start(self):
        """ Run the infinite i/o event loop.
        """
        while True:
            # with do_exit Event set (being thread or process),
            # wait to event set to exit, ignoring KeyboardInterrupt.
            try:
                if self.do_exit.is_set():
                    close_all_sockets(self._socket_map)
                    break
                self._loop()
            except (KeyboardInterrupt, SystemExit):
                close_all_sockets(self._socket_map)
                raise

    def stop(self):
        """ Signalize the gaplic instance to stop.
        """
        self.do_exit.set()

    def mt_subprocess(self):
        """ Subclass :class:`GAplic` class and override this function
        to do extra work in the infinite loop.
        """

    def enqueue_event(self, event):
        """ Post the event in the next :term:`gaplic` loop cycle,
        not right now.

        :param event: :term:`event` to send.
        :param destination: destination :term:`gobj` of the event.
        :param kw: keyword argument with data associated to event.

            .. note::
                All the keyword arguments **are added as attributes** to
                the sent :term:`event`.

        Same as :meth:`send_event` function but the event is sent in the
        next :term:`gaplic` loop cycle, not right now.

        It **does not return** the return of the executed action because the
        action it's executed later, in the next loop cycle.

        It's mandatory use this function, if the `destination`
        :term:`gobj` is not local.

        .. note:: It **DOES NOT** return the return of the executed action
            because the action it's executed later, in the next loop cycle,
            so you **CANNOT** receive valid direct data from the action.

        .. warning:: If you use :meth:`post_event` without a :term:`gaplic`
            then a :exc:`GAplicError` exception will be raised.

        ``destination`` must be a `string` or :class:`GObj` types, otherwise a
        :exc:`GObjError` will be raised.

        ``event`` must be a `string` or :class:`Event` types, otherwise a
        :exc:`EventError` will be raised.

        If ``event`` is an :class:`Event` instance, a new :class:`Event`
        duplicated instance is returned, but it will be updated with
        the new ``destination`` and ``kw`` keyword arguments.

        .. note::
            All the keyword arguments used in the factory function
            **are added as attributes** to the created :term:`event` instance.
            You must consult the attributes supported by each machine's event.
        """
        self._qevent.append(event)

    def _process_qevent(self):
        """ Return True if remains events.
        """
        # ln = len(self._qevent)
        # print "qevent...........%d" % (ln)
        it = 0
        maximum = 10
        while True:
            if it > maximum:
                # balance the work
                return True
            try:
                event = self._qevent.popleft()
            except IndexError:
                break
            else:
                it += 1
                # To send to external gaplics, destination must be 'router'
                try:
                    destination = self._resolv_destination(event.destination)
                    cur_ident = threading.current_thread().ident
                    if cur_ident != self._thread_ident:
                        if self.logger:
                            self.logger.error("??????????????????")

                    dst_ident = destination.gaplic._thread_ident
                    if cur_ident == dst_ident:
                        self.send_event(
                            destination,
                            event.event_name,
                            **event.kw)
                    else:
                        # Yeah, send to another gaplic
                        gaplic = get_gaplic_by_thread_ident(dst_ident)
                        if gaplic:
                            gaplic.enqueue_event(event)
                except Exception as e:
                    if self.logger:
                        self.logger.exception("ERROR _process_qevent: %s" % e)

        return False

    def _process_timer(self):
        # don't use iteritems() items(),
        # some xtimer can be removed during processing timers
        some_event = False
        try:
            for xtimer in iterkeys_(self._gotter_timers):
                try:
                    value = self._gotter_timers[xtimer]
                except KeyError:
                    # timer deleted while loop.
                    continue
                some_event = True
                if value and _elapsed_timer(value):
                    if xtimer.autostart:
                        next_run = _start_timer(xtimer.sec)
                        xtimer.next_run = next_run
                        self._gotter_timers[xtimer] = next_run
                    else:
                        self._gotter_timers[xtimer] = 0
                        xtimer.next_run = 0
                    if xtimer.timer_callback:
                        xtimer.timer_callback()
                    if not xtimer.autostart:
                        self._gotter_timers.pop(xtimer)
        except RuntimeError:
            # timer deleted while loop.
            some_event = True
        return some_event

    def _setTimeout(self, sec, timer_callback, autostart=False):
        """ Set a callback to be executed in ``sec`` seconds.
        Function used by :class:`GTimer` gobj. Not for general use.
        Return an object to be used in :func:`clearTimeout`.
        """
        xtimer = _XTimer(sec, timer_callback, autostart)
        next_run = _start_timer(sec)
        xtimer.next_run = next_run
        self._gotter_timers[xtimer] = next_run
        return xtimer

    def _clearTimeout(self, xtimer):
        """ Clear callback timeout.
        Function used by :class:`GTimer` gobj. Not for general use.
        """
        t = self._gotter_timers.get(xtimer, None)
        if t is not None:
            # prevent timer cleared in proces_timer loop
            self._gotter_timers.pop(xtimer)
            xtimer.next_run = 0
            xtimer.timer_callback = None

    def add_timeout(self, deadline, callback):
        """ Compatible with tornado.io_loop

        ``deadline`` only seconds please.

        Calls the given callback at the time deadline from the I/O loop.

        Returns a handle that may be passed to remove_timeout to cancel.

        ``deadline`` may be a number denoting a unix timestamp (as returned
        by ``time.time()`` or a ``datetime.timedelta`` object for a deadline
        relative to the current time.

        Note that it is not safe to call `add_timeout` from other threads.
        Instead, you must use `add_callback` to transfer control to the
        IOLoop's thread, and then call `add_timeout` from there.
        """
        xtimer = self._setTimeout(deadline, callback)
        return xtimer

    def remove_timeout(self, xtimer):
        """ Compatible with tornado.io_loop

        Cancels a pending timeout.

        The argument is a handle as returned by add_timeout.
        """
        # Removing from a heap is complicated, so just leave the defunct
        # timeout object in the queue (see discussion in
        # http://docs.python.org/library/heapq.html).
        # If this turns out to be a problem, we could add a garbage
        # collection pass whenever there are too many dead timeouts.
        xtimer.timer_callback = None
        self._clearTimeout(xtimer)

    def add_callback(self, callback, *args, **kwargs):
        """ Call the given callback in the next I/O loop iteration.
        """
        list_empty = not self._callbacks
        deferred = Deferred(0, callback, *args, **kwargs)
        self._callbacks.append(deferred)
        if list_empty:
            if threading.current_thread().ident != self._thread_ident:
                # If we're in the IOLoop's thread, we know it's not currently
                # polling.  If we're not, and we added the first callback to an
                # empty list, we may need to wake it up (it may wake up on its
                # own, but an occasional extra wake is harmless).  Waking
                # up a polling IOLoop is relatively expensive, so we try to
                # avoid it when we can.
                pass
                # TODO: study this from tornado
                # self._waker.wake()

    def _run_callback(self, deferred):
        try:
            deferred()
        except Exception:
            self.handle_callback_exception(deferred)

    def handle_callback_exception(self, callback):
        """This method is called whenever a callback run by the IOLoop
        throws an exception.

        By default simply logs the exception as an error.  Subclasses
        may override this method to customize reporting of exceptions.

        The exception itself is not passed explicitly, but is available
        in sys.exc_info.
        """
        if self.logger:
            self.logger.error(
                "Exception in callback %r",
                callback,
                exc_info=True)


#===============================================================
#                   Thread wrapper for gaplic
#===============================================================
class GAplicThreadWorker(threading.Thread):
    """ Class derived from :class:`threading.Thread` to run gaplic
    in thread environment.
    """
    def __init__(self, gaplic):
        threading.Thread.__init__(self)
        self.daemon = True
        self.gaplic = gaplic

    def run(self):
        """ Override the :meth:`threading.Thread.run` method.

        Run the gaplic loop in a separate thread.
        """
        self.gaplic.start()

    def stop(self):
        """ Stop the worker.
        """
        self.gaplic.stop()

    def join(self, timeout=10.0):   # wait until 10 seconds for thread killed.
        """ Wait for worker to stop, until ``timeout`` seconds."""
        super(GAplicThreadWorker, self).join(timeout)


def setup_gaplic_thread(gaplic):
    """ Run gaplic as thread.
    Return the worker.
    You must call worker.start() to run the thread.
    """
    worker = GAplicThreadWorker(gaplic)
    add_global_thread(worker)
    return worker

#===============================================================
#                   Process wrapper for gaplic
#===============================================================
import multiprocessing


class GAplicProcessWorker(multiprocessing.Process):
    """ Class derived from :class:`multiprocessing.Process` to run gaplic
    in subprocess environment.
    """
    def __init__(self, gaplic):
        multiprocessing.Process.__init__(self)
        self.daemon = True
        self.gaplic = gaplic

    def run(self):
        """ Override the :meth:`multiprocessing.Process.run` method.

        Run the gaplic loop in a separate process.
        """
        self.gaplic.start()

    def stop(self):
        """ Stop the worker.
        """
        self.gaplic.stop()

    def join(self, timeout=10.0):   # wait until 10 seconds for process killed.
        """ Wait for worker to stop, until ``timeout`` seconds."""
        super(GAplicProcessWorker, self).join(timeout)


def setup_gaplic_process(gaplic):
    """ Run gaplic as process.
    Return the worker.
    You must call worker.start() to run the subprocess.
    """
    worker = GAplicProcessWorker(gaplic)
    add_global_subprocess(worker)
    return worker


def gaplic_factory(loader, global_conf, **local_conf):
    """ To use with PasteDeploy in composite.

        Items of *composite* section:

        :main: name of a section that must return a :term:`gaplic`
               instance. It will be the **principal** :term:`gaplic`.

        :threads: name of sections that must return :term:`gaplic`
               instances. They will run in threads.

        :subprocesses: name of sections that must return :term:`gaplic`
               instances. They will run in subprocesses.

        :wsgi: name of sections that must return a *app paste factory*.
            Wsgi applications are saved as global apps (set_global_app()).


        Example::

            [composite:main]
            use = call:ginsfsm.gaplic:gaplic_factory
            main = wsgi-server
            wsgi = wsgi-application

            [app:wsgi-server]
            use = call:ginsfsm.examples.wsgi.simple_wsgi_server:main
            host = 0.0.0.0
            port = 8000
            application = wsgi-application
            GSock.trace_dump = true
            GObj.trace_mach = true

            [app:wsgi-application]
            use=call:ginsfsm.examples.wsgi.simple_wsgi_server:paste_app_factory


    The prototype for ``wsgi`` (paste app factory) is::

        def paste_app_factory(global_conf, **local_conf):
            return wsgi-application

    The prototype for ``main``, ``threads`` and ``subprocesses`` is::

        def main(global_conf, **local_config):
            return gaplic-instance

    """
    main = local_conf.get('main')
    wsgis = local_conf.get('wsgi', '').split()
    threads = local_conf.get('threads', '').split()
    subprocesses = local_conf.get('subprocesses', '').split()

    # Firstly create main gaplic, to can pass it to wsgi's
    main_gaplic = loader.get_app(main, global_conf=global_conf)
    set_global_main_gaplic(main_gaplic)

    for thread in threads:
        gaplic = loader.get_app(thread, global_conf=global_conf)
        worker = setup_gaplic_thread(gaplic)
        worker.start()

    for subprocess in subprocesses:
        gaplic = loader.get_app(subprocess, global_conf=global_conf)
        worker = setup_gaplic_process(gaplic)
        worker.start()

    for wsgi in wsgis:
        app = loader.get_app(wsgi, global_conf=global_conf)
        set_global_app(wsgi, app)

    return main_gaplic



# -*- encoding: utf-8 -*-
""" GSorteo GObj

.. autoclass:: GSorteo
    :members: start_up

"""

from ginsfsm.gobj import GObj
from ginsfsm.c_timer import GTimer


def ac_regalo(self, event):
    """ Event attributes:
        * :attr:`sorteo`: .

    """
    user_name = event.user_name

    if event.__intra_event__:
        origin_gaplic = event.__intra_event__.origin_gaplic
        origin_role = event.__intra_event__.origin_role
        origin_gobj = event.__intra_event__.origin_gobj
        self.user_sorteo[user_name] = {
            'gaplic': origin_gaplic,
            'role': origin_role,
            'gobj': origin_gobj
        }

    self.set_timeout(3)


def ac_sorteo(self, event):
    self.broadcast_event('EV_REGALO', regalo="Premio gordo!!")
    self.set_timeout(3)


GSORTEO_FSM = {
    'event_list': (
        'EV_REGALO:top input output',
        'EV_TIMEOUT:bottom input',
    ),
    'state_list': (
        'ST_IDLE',
    ),
    'machine': {
        'ST_IDLE':
        (
            ('EV_REGALO',      ac_regalo,      None),
            ('EV_TIMEOUT',     ac_sorteo,      None),
        ),
    }
}

GSORTEO_GCONFIG = {
    'subscriber': [
        None, None, 0, None,
        "subcriber of all output-events."
        "Default is ``None``, i.e., the parent"
    ],
}


class GSorteo(GObj):
    """  GSorteo GObj.
    A Sample gobj.

    .. ginsfsm::
       :fsm: GSORTEO_FSM
       :gconfig: GSORTEO_GCONFIG

    *Input-Events:*
        * :attr:`'EV_REGALO'`: sample input event.

          Event attributes:

              * ``data``: sample event attribute.

    *Output-Events:*
        * :attr:`'EV_REGALO_ACK'`: sample output event.

          Event attributes:

              * ``data``: sample event attribute.

    """
    def __init__(self):
        GObj.__init__(self, GSORTEO_FSM, GSORTEO_GCONFIG)
        self.user_sorteo = {}

    def start_up(self):
        """ Initialization zone.
        """
        self.timer = self.create_gobj(
            None,
            GTimer,
            self,
            timeout_event_name='EV_TIMEOUT',
        )
        self.set_timeout(3)

    def set_timeout(self, seconds):
        self.send_event(self.timer, 'EV_SET_TIMER', seconds=seconds)

    def clear_timeout(self):
        self.send_event(self.timer, 'EV_SET_TIMER', seconds=-1)



# -*- encoding: utf-8 -*-
"""
GObj :class:`GWsgiServer`
=========================

WSGI server.

It uses :class:`ginsfsm.protocols.http.server.c_http_server.GHttpServer`.

.. autoclass:: GWsgiServer
    :members:

"""

from ginsfsm.gobj import GObj
from ginsfsm.compat import string_types
from ginsfsm.globals import get_global_app
from ginsfsm.protocols.http.server.c_http_server import GHttpServer
from ginsfsm.protocols.wsgi.common.wsgi_response import WsgiResponse
from ginsfsm.protocols.http.server.c_http_server import GHTTPSERVER_GCONFIG


def ac_channel_opened(self, event):
    """ New client opened.
    """


def ac_channel_closed(self, event):
    """ Client closed.
    """


def ac_request(self, event):
    # quiza es mejor generalizar este nivel.
    # elegir el tipo de aplicación:
    # si es un GObj enviarle el evento, sino, por defecto, sería una wsgi.
    # el evento de desconexión lo tengo solucionado: un subscribe al channel.

    channel = event.channel
    request = event.request
    application = self.select_app(request)
    response = WsgiResponse(request, self, application)
    self.send_event(channel, 'EV_HTTP_RESPONSE', response=response)


GWSGISERVER_FSM = {
    'event_list': (
        'EV_HTTP_CHANNEL_OPENED: bottom input',
        'EV_HTTP_CHANNEL_CLOSED: bottom input',
        'EV_HTTP_REQUEST: bottom input',
    ),
    'state_list': ('ST_IDLE',),
    'machine': {
        'ST_IDLE':
        (
            ('EV_HTTP_CHANNEL_OPENED',  ac_channel_opened,  None),
            ('EV_HTTP_CHANNEL_CLOSED',  ac_channel_closed,  None),
            ('EV_HTTP_REQUEST',         ac_request,         None),
        ),
    }
}

GWSGISERVER_GCONFIG = GHTTPSERVER_GCONFIG.copy()
GWSGISERVER_GCONFIG.update({
    'application': [None, None, 0, None, "wsgi application"],
})


class GWsgiServer(GObj):
    """  WSGI Server gobj.

    .. ginsfsm::
       :fsm: GWSGISERVER_FSM
       :gconfig: GWSGISERVER_GCONFIG

    *Bottom Input-Events:*

        * :attr:`'EV_HTTP_CHANNEL_OPENED'`: new http client.

          Event attributes:

            * ``channel``: http channel.

        * :attr:`'EV_HTTP_CHANNEL_CLOSED'`: http client closed.

          Event attributes:

            * ``channel``: http channel.

        * :attr:`'EV_HTTP_REQUEST'`: new http request.

          Event attributes:

            * ``channel``: http channel.
            * ``request``: http request.

        See :class:`ginsfsm.protocols.http.server.c_http_clisrv.GHttpCliSrv`.

    """

    def __init__(self):
        GObj.__init__(self, GWSGISERVER_FSM, GWSGISERVER_GCONFIG)
        self._n_connected_clisrv = 0

    def start_up(self):
        self.ghttpserver = self.create_gobj(
            self.name if self.name else 'http-server',
            GHttpServer,
            self,
            subscriber=self,
            host=self.config.host,
            port=self.config.port,
            origins=self.config.origins,
            inactivity_timeout=self.config.inactivity_timeout,
            responseless_timeout=self.config.responseless_timeout,
            maximum_simultaneous_requests=
                self.config.maximum_simultaneous_requests,
        )
        self.serversock = self.ghttpserver.gserversock.socket
        # Used in environ
        self.effective_host, self.effective_port = self.getsockname()
        self.server_name = self._get_server_name(self.config.host)

    def select_app(self, request):
        # Starting from ini file, the application references is a string
        # because the wsgi-apps are loaded later than main gaplic.
        if isinstance(self.config.application, string_types):
            self.config.application = get_global_app(self.config.application)
        return self.config.application

    def _get_server_name(self, ip):
        """Given an IP or hostname, try to determine the server name."""
        if ip:
            srv_name = str(ip)
        else:
            srv_name = str(self.serversock.socketmod.gethostname())
        # Convert to a host name if necessary.
        for c in srv_name:
            if c != '.' and not c.isdigit():
                return srv_name
        try:
            if srv_name == '0.0.0.0':
                return 'localhost'
            srv_name = self.serversock.socketmod.gethostbyaddr(srv_name)[0]
        except:
            pass
        return srv_name

    def getsockname(self):
        return self.serversock.getsockname()



# (c) 2005 Ian Bicking and contributors; written for Paste
# (http://pythonpaste.org) Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license.php
#
# For discussion of daemonizing:
# http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/278731
#
# Code taken also from QP: http://www.mems-exchange.org/software/qp/ From
# lib/site.py

import atexit
import ctypes
import errno
import logging
import optparse
import os
import re
import subprocess
import sys
import textwrap
import threading
import time
import traceback

from paste.deploy import loadapp

from ginsfsm.compat import WIN

from ginsfsm.paster import setup_logging

MAXFD = 1024

if WIN and not hasattr(os, 'kill'):  # pragma: no cover
    # py 2.6 on windows
    def kill(pid, sig=None):
        """kill function for Win32"""
        # signal is ignored, semibogus raise message
        kernel32 = ctypes.windll.kernel32
        handle = kernel32.OpenProcess(1, 0, pid)
        if (0 == kernel32.TerminateProcess(handle, 0)):
            raise OSError('No such process %s' % pid)
else:
    kill = os.kill


def main(argv=sys.argv, quiet=False):
    command = PServeCommand(argv, quiet=quiet)
    return command.run()


class DaemonizeException(Exception):
    pass


class PServeCommand(object):

    usage = '%prog config_uri [start|stop|restart|status] [var=value]'
    description = """\
    This command serves a web application that uses a PasteDeploy
    configuration file for the application.

    If start/stop/restart is given, then --daemon is implied, and it will
    start (normal operation), stop (--stop-daemon), or do both.

    You can also include variable assignments like 'http_port=8080'
    and then use %(http_port)s in your config files.
    """
    verbose = 1

    parser = optparse.OptionParser(
        usage,
        description=textwrap.dedent(description)
        )
    parser.add_option(
        '-n', '--app-name',
        dest='app_name',
        metavar='NAME',
        help="Load the named application (default main)")
    if hasattr(os, 'fork'):
        parser.add_option(
            '--daemon',
            dest="daemon",
            action="store_true",
            help="Run in daemon (background) mode")
    parser.add_option(
        '--pid-file',
        dest='pid_file',
        metavar='FILENAME',
        help=("Save PID to file (default to ginsfsm.pid if running in "
              "daemon mode)"))
    parser.add_option(
        '--log-file',
        dest='log_file',
        metavar='LOG_FILE',
        help="Save output to the given log file (redirects stdout)")
    parser.add_option(
        '--reload',
        dest='reload',
        action='store_true',
        help="Use auto-restart file monitor")
    parser.add_option(
        '--reload-interval',
        dest='reload_interval',
        default=1,
        help=("Seconds between checking files (low number can cause "
              "significant CPU usage)"))
    parser.add_option(
        '--monitor-restart',
        dest='monitor_restart',
        action='store_true',
        help="Auto-restart application if it dies")
    parser.add_option(
        '--status',
        action='store_true',
        dest='show_status',
        help="Show the status of the (presumably daemonized) application")

    if hasattr(os, 'setuid'):
        # I don't think these are available on Windows
        parser.add_option(
            '--user',
            dest='set_user',
            metavar="USERNAME",
            help="Set the user (usually only possible when run as root)")
        parser.add_option(
            '--group',
            dest='set_group',
            metavar="GROUP",
            help="Set the group (usually only possible when run as root)")

    parser.add_option(
        '--stop-daemon',
        dest='stop_daemon',
        action='store_true',
        help=('Stop a daemonized application (given a PID file, or default '
              'ginsfsm.pid file)'))

    _scheme_re = re.compile(r'^[a-z][a-z]+:', re.I)

    default_verbosity = 1

    _reloader_environ_key = 'PYTHON_RELOADER_SHOULD_RUN'
    _monitor_environ_key = 'PASTE_MONITOR_SHOULD_RUN'

    possible_subcommands = ('start', 'stop', 'restart', 'status')

    def __init__(self, argv, quiet=False):
        self.quiet = quiet
        self.options, self.args = self.parser.parse_args(argv[1:])

    def out(self, msg):  # pragma: no cover
        if not self.quiet:
            print(msg)

    def run(self):  # pragma: no cover
        if self.options.stop_daemon:
            return self.stop_daemon()

        if not hasattr(self.options, 'set_user'):
            # Windows case:
            self.options.set_user = self.options.set_group = None

        # @@: Is this the right stage to set the user at?
        self.change_user_group(
            self.options.set_user, self.options.set_group)

        if not self.args:
            self.out('You must give a config file')
            return 2
        app_spec = self.args[0]
        if (len(self.args) > 1
            and self.args[1] in self.possible_subcommands):
            cmd = self.args[1]
            restvars = self.args[2:]
        else:
            cmd = None
            restvars = self.args[1:]

        if self.options.reload:
            if os.environ.get(self._reloader_environ_key):
                if self.verbose > 1:
                    self.out('Running reloading file monitor')
                install_reloader(int(self.options.reload_interval), [app_spec])
                # if self.requires_config_file:
                #     watch_file(self.args[0])
            else:
                return self.restart_with_reloader()

        if cmd not in (None, 'start', 'stop', 'restart', 'status'):
            self.out(
                'Error: must give start|stop|restart (not %s)' % cmd)
            return 2

        if cmd == 'status' or self.options.show_status:
            return self.show_status()

        if cmd == 'restart' or cmd == 'stop':
            result = self.stop_daemon()
            if result:
                if cmd == 'restart':
                    self.out("Could not stop daemon; aborting")
                else:
                    self.out("Could not stop daemon")
                return result
            if cmd == 'stop':
                return result
            self.options.daemon = True

        if cmd == 'start':
            self.options.daemon = True

        app_name = self.options.app_name
        vars = self.parse_vars(restvars)
        if not self._scheme_re.search(app_spec):
            app_spec = 'config:' + app_spec
        base = os.getcwd()

        if getattr(self.options, 'daemon', False):
            if not self.options.pid_file:
                self.options.pid_file = 'ginsfsm.pid'
            if not self.options.log_file:
                self.options.log_file = 'ginsfsm.log'

        # Ensure the log file is writeable
        if self.options.log_file:
            try:
                writeable_log_file = open(self.options.log_file, 'a')
            except IOError as ioe:
                msg = 'Error: Unable to write to log file: %s' % ioe
                raise ValueError(msg)
            writeable_log_file.close()

        # Ensure the pid file is writeable
        if self.options.pid_file:
            try:
                writeable_pid_file = open(self.options.pid_file, 'a')
            except IOError as ioe:
                msg = 'Error: Unable to write to pid file: %s' % ioe
                raise ValueError(msg)
            writeable_pid_file.close()

        if getattr(self.options, 'daemon', False):
            try:
                self.daemonize()
            except DaemonizeException as ex:
                if self.verbose > 0:
                    self.out(str(ex))
                return 2

        if (self.options.monitor_restart
            and not os.environ.get(self._monitor_environ_key)):
            return self.restart_with_monitor()

        if self.options.pid_file:
            self.record_pid(self.options.pid_file)

        if self.options.log_file:
            stdout_log = LazyWriter(self.options.log_file, 'a')
            sys.stdout = stdout_log
            sys.stderr = stdout_log
            logging.basicConfig(stream=stdout_log)

        log_fn = app_spec
        if log_fn.startswith('config:'):
            log_fn = app_spec[len('config:'):]
        elif log_fn.startswith('egg:'):
            log_fn = None
        if log_fn:
            log_fn = os.path.join(base, log_fn)
            setup_logging(log_fn)

        app = self.loadapp(app_spec, name=app_name,
                           relative_to=base, global_conf=vars)

        if self.verbose > 0:
            if hasattr(os, 'getpid'):
                msg = 'Starting application in PID %i.' % os.getpid()
            else:
                msg = 'Starting application.'
            self.out(msg)

        def serve():
            try:
                app.start()
            except (SystemExit, KeyboardInterrupt) as e:
                if self.verbose > 1:
                    raise
                if str(e):
                    msg = ' ' + str(e)
                else:
                    msg = ''
                self.out('Exiting%s (-v to see traceback)' % msg)

        serve()

    def loadapp(self, app_spec, name, relative_to, **kw):  # pragma: no cover
        return loadapp(app_spec, name=name, relative_to=relative_to, **kw)

    def parse_vars(self, args):
        """
        Given variables like ``['a=b', 'c=d']`` turns it into ``{'a':
        'b', 'c': 'd'}``
        """
        result = {}
        for arg in args:
            if '=' not in arg:
                raise ValueError(
                    'Variable assignment %r invalid (no "=")'
                    % arg)
            name, value = arg.split('=', 1)
            result[name] = value
        return result

    def quote_first_command_arg(self, arg):  # pragma: no cover
        """
        There's a bug in Windows when running an executable that's
        located inside a path with a space in it.  This method handles
        that case, or on non-Windows systems or an executable with no
        spaces, it just leaves well enough alone.
        """
        if (sys.platform != 'win32' or ' ' not in arg):
            # Problem does not apply:
            return arg
        try:
            import win32api
        except ImportError:
            raise ValueError(
                "The executable %r contains a space, and in order to "
                "handle this issue you must have the win32api module "
                "installed" % arg)
        arg = win32api.GetShortPathName(arg)
        return arg

    def daemonize(self):  # pragma: no cover
        pid = live_pidfile(self.options.pid_file)
        if pid:
            raise DaemonizeException(
                "Daemon is already running (PID: %s from PID file %s)"
                % (pid, self.options.pid_file))

        if self.verbose > 0:
            self.out('Entering daemon mode')
        pid = os.fork()
        if pid:
            # The forked process also has a handle on resources, so we
            # *don't* want proper termination of the process, we just
            # want to exit quick (which os._exit() does)
            os._exit(0)
        # Make this the session leader
        os.setsid()
        # Fork again for good measure!
        pid = os.fork()
        if pid:
            os._exit(0)

        # @@: Should we set the umask and cwd now?

        import resource  # Resource usage information.
        maxfd = resource.getrlimit(resource.RLIMIT_NOFILE)[1]
        if (maxfd == resource.RLIM_INFINITY):
            maxfd = MAXFD
        # Iterate through and close all file descriptors.
        for fd in range(0, maxfd):
            try:
                os.close(fd)
            except OSError:  # ERROR, fd wasn't open to begin with (ignored)
                pass

        if (hasattr(os, "devnull")):
            REDIRECT_TO = os.devnull
        else:
            REDIRECT_TO = "/dev/null"
        os.open(REDIRECT_TO, os.O_RDWR)  # standard input (0)
        # Duplicate standard input to standard output and standard error.
        os.dup2(0, 1)  # standard output (1)
        os.dup2(0, 2)  # standard error (2)

    def _remove_pid_file(self, written_pid, filename, verbosity):
        current_pid = os.getpid()
        if written_pid != current_pid:
            # A forked process must be exiting, not the process that
            # wrote the PID file
            return
        if not os.path.exists(filename):
            return
        with open(filename) as f:
            content = f.read().strip()
        try:
            pid_in_file = int(content)
        except ValueError:
            pass
        else:
            if pid_in_file != current_pid:
                msg = "PID file %s contains %s, not expected PID %s"
                self.out(msg % (filename, pid_in_file, current_pid))
                return
        if verbosity > 0:
            self.out("Removing PID file %s" % filename)
        try:
            os.unlink(filename)
            return
        except OSError as e:
            # Record, but don't give traceback
            self.out("Cannot remove PID file: (%s)" % e)
        # well, at least lets not leave the invalid PID around...
        try:
            with open(filename, 'w') as f:
                f.write('')
        except OSError as e:
            self.out('Stale PID left in file: %s (%s)' % (filename, e))
        else:
            self.out('Stale PID removed')

    def record_pid(self, pid_file):
        pid = os.getpid()
        if self.verbose > 1:
            self.out('Writing PID %s to %s' % (pid, pid_file))
        with open(pid_file, 'w') as f:
            f.write(str(pid))
        atexit.register(self._remove_pid_file, pid, pid_file, self.verbose)

    def stop_daemon(self):  # pragma: no cover
        pid_file = self.options.pid_file or 'ginsfsm.pid'
        if not os.path.exists(pid_file):
            self.out('No PID file exists in %s' % pid_file)
            return 1
        pid = read_pidfile(pid_file)
        if not pid:
            self.out("Not a valid PID file in %s" % pid_file)
            return 1
        pid = live_pidfile(pid_file)
        if not pid:
            self.out("PID in %s is not valid (deleting)" % pid_file)
            try:
                os.unlink(pid_file)
            except (OSError, IOError) as e:
                self.out("Could not delete: %s" % e)
                return 2
            return 1
        for j in range(10):
            if not live_pidfile(pid_file):
                break
            import signal
            kill(pid, signal.SIGTERM)
            time.sleep(1)
        else:
            self.out("failed to kill web process %s" % pid)
            return 3
        if os.path.exists(pid_file):
            os.unlink(pid_file)
        return 0

    def show_status(self):  # pragma: no cover
        pid_file = self.options.pid_file or 'ginsfsm.pid'
        if not os.path.exists(pid_file):
            self.out('No PID file %s' % pid_file)
            return 1
        pid = read_pidfile(pid_file)
        if not pid:
            self.out('No PID in file %s' % pid_file)
            return 1
        pid = live_pidfile(pid_file)
        if not pid:
            self.out('PID %s in %s is not running' % (pid, pid_file))
            return 1
        self.out('Server running in PID %s' % pid)
        return 0

    def restart_with_reloader(self):  # pragma: no cover
        self.restart_with_monitor(reloader=True)

    def restart_with_monitor(self, reloader=False):  # pragma: no cover
        if self.verbose > 0:
            if reloader:
                self.out('Starting subprocess with file monitor')
            else:
                self.out('Starting subprocess with monitor parent')
        while 1:
            args = [self.quote_first_command_arg(sys.executable)] + sys.argv
            new_environ = os.environ.copy()
            if reloader:
                new_environ[self._reloader_environ_key] = 'true'
            else:
                new_environ[self._monitor_environ_key] = 'true'
            proc = None
            try:
                try:
                    _turn_sigterm_into_systemexit()
                    proc = subprocess.Popen(args, env=new_environ)
                    exit_code = proc.wait()
                    proc = None
                except KeyboardInterrupt:
                    self.out('^C caught in monitor process')
                    if self.verbose > 1:
                        raise
                    return 1
            finally:
                if proc is not None:
                    import signal
                    try:
                        kill(proc.pid, signal.SIGTERM)
                    except (OSError, IOError):
                        pass

            if reloader:
                # Reloader always exits with code 3; but if we are
                # a monitor, any exit code will restart
                if exit_code != 3:
                    return exit_code
            if self.verbose > 0:
                self.out('%s %s %s' % ('-' * 20, 'Restarting', '-' * 20))

    def change_user_group(self, user, group):  # pragma: no cover
        if not user and not group:
            return
        import pwd
        import grp
        uid = gid = None
        if group:
            try:
                gid = int(group)
                group = grp.getgrgid(gid).gr_name
            except ValueError:
                import grp
                try:
                    entry = grp.getgrnam(group)
                except KeyError:
                    raise ValueError(
                        "Bad group: %r; no such group exists" % group)
                gid = entry.gr_gid
        try:
            uid = int(user)
            user = pwd.getpwuid(uid).pw_name
        except ValueError:
            try:
                entry = pwd.getpwnam(user)
            except KeyError:
                raise ValueError(
                    "Bad username: %r; no such user exists" % user)
            if not gid:
                gid = entry.pw_gid
            uid = entry.pw_uid
        if self.verbose > 0:
            self.out('Changing user to %s:%s (%s:%s)' % (
                user, group or '(unknown)', uid, gid))
        if gid:
            os.setgid(gid)
        if uid:
            os.setuid(uid)


class LazyWriter(object):

    """
    File-like object that opens a file lazily when it is first written
    to.
    """

    def __init__(self, filename, mode='w'):
        self.filename = filename
        self.fileobj = None
        self.lock = threading.Lock()
        self.mode = mode

    def open(self):
        if self.fileobj is None:
            self.lock.acquire()
            try:
                if self.fileobj is None:
                    self.fileobj = open(self.filename, self.mode)
            finally:
                self.lock.release()
        return self.fileobj

    def close(self):
        fileobj = self.fileobj
        if fileobj is not None:
            fileobj.close()

    def __del__(self):
        self.close()

    def write(self, text):
        fileobj = self.open()
        fileobj.write(text)
        fileobj.flush()

    def writelines(self, text):
        fileobj = self.open()
        fileobj.writelines(text)
        fileobj.flush()

    def flush(self):
        self.open().flush()


def live_pidfile(pidfile):  # pragma: no cover
    """(pidfile:str) -> int | None
    Returns an int found in the named file, if there is one,
    and if there is a running process with that process id.
    Return None if no such process exists.
    """
    pid = read_pidfile(pidfile)
    if pid:
        try:
            kill(int(pid), 0)
            return pid
        except OSError as e:
            if e.errno == errno.EPERM:
                return pid
    return None


def read_pidfile(filename):
    if os.path.exists(filename):
        try:
            with open(filename) as f:
                content = f.read()
            return int(content.strip())
        except (ValueError, IOError):
            return None
    else:
        return None


def ensure_port_cleanup(
    bound_addresses, maxtries=30, sleeptime=2):  # pragma: no cover
    """
    This makes sure any open ports are closed.

    Does this by connecting to them until they give connection
    refused.  Servers should call like::

        ensure_port_cleanup([80, 443])
    """
    atexit.register(_cleanup_ports, bound_addresses, maxtries=maxtries,
                    sleeptime=sleeptime)


def _cleanup_ports(
    bound_addresses, maxtries=30, sleeptime=2):  # pragma: no cover
    # Wait for the server to bind to the port.
    import socket
    import errno
    for bound_address in bound_addresses:
        for attempt in range(maxtries):
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            try:
                sock.connect(bound_address)
            except socket.error as e:
                if e.args[0] != errno.ECONNREFUSED:
                    raise
                break
            else:
                time.sleep(sleeptime)
        else:
            raise SystemExit('Timeout waiting for port.')
        sock.close()


def _turn_sigterm_into_systemexit():  # pragma: no cover
    """
    Attempts to turn a SIGTERM exception into a SystemExit exception.
    """
    try:
        import signal
    except ImportError:
        return

    def handle_term(signo, frame):
        raise SystemExit
    signal.signal(signal.SIGTERM, handle_term)


def install_reloader(poll_interval=1, extra_files=None):  # pragma: no cover
    """
    Install the reloading monitor.

    On some platforms server threads may not terminate when the main
    thread does, causing ports to remain open/locked.  The
    ``raise_keyboard_interrupt`` option creates a unignorable signal
    which causes the whole application to shut-down (rudely).
    """
    mon = Monitor(poll_interval=poll_interval)
    if extra_files is None:
        extra_files = []
    mon.extra_files.extend(extra_files)
    t = threading.Thread(target=mon.periodic_reload)
    t.setDaemon(True)
    t.start()


class classinstancemethod(object):
    """
    Acts like a class method when called from a class, like an
    instance method when called by an instance.  The method should
    take two arguments, 'self' and 'cls'; one of these will be None
    depending on how the method was called.
    """

    def __init__(self, func):
        self.func = func
        self.__doc__ = func.__doc__

    def __get__(self, obj, type=None):
        return _methodwrapper(self.func, obj=obj, type=type)


class _methodwrapper(object):

    def __init__(self, func, obj, type):
        self.func = func
        self.obj = obj
        self.type = type

    def __call__(self, *args, **kw):
        assert not 'self' in kw and not 'cls' in kw, (
            "You cannot use 'self' or 'cls' arguments to a "
            "classinstancemethod")
        return self.func(*((self.obj, self.type) + args), **kw)


class Monitor(object):  # pragma: no cover
    """
    A file monitor and server restarter.

    Use this like:

    ..code-block:: Python

        install_reloader()

    Then make sure your server is installed with a shell script like::

        err=3
        while test "$err" -eq 3 ; do
            python server.py
            err="$?"
        done

    or is run from this .bat file (if you use Windows)::

        @echo off
        :repeat
            python server.py
        if %errorlevel% == 3 goto repeat

    or run a monitoring process in Python (``pserve --reload`` does
    this).

    Use the ``watch_file(filename)`` function to cause a reload/restart for
    other other non-Python files (e.g., configuration files).  If you have
    a dynamic set of files that grows over time you can use something like::

        def watch_config_files():
            return CONFIG_FILE_CACHE.keys()
        add_file_callback(watch_config_files)

    Then every time the reloader polls files it will call
    ``watch_config_files`` and check all the filenames it returns.
    """
    instances = []
    global_extra_files = []
    global_file_callbacks = []

    def __init__(self, poll_interval):
        self.module_mtimes = {}
        self.keep_running = True
        self.poll_interval = poll_interval
        self.extra_files = list(self.global_extra_files)
        self.instances.append(self)
        self.file_callbacks = list(self.global_file_callbacks)

    def _exit(self):
        # use os._exit() here and not sys.exit() since within a
        # thread sys.exit() just closes the given thread and
        # won't kill the process; note os._exit does not call
        # any atexit callbacks, nor does it do finally blocks,
        # flush open files, etc.  In otherwords, it is rude.
        os._exit(3)

    def periodic_reload(self):
        while True:
            if not self.check_reload():
                self._exit()
                break
            time.sleep(self.poll_interval)

    def check_reload(self):
        filenames = list(self.extra_files)
        for file_callback in self.file_callbacks:
            try:
                filenames.extend(file_callback())
            except:
                print(
                    "Error calling reloader callback %r:" % file_callback)
                traceback.print_exc()
        for module in sys.modules.values():
            try:
                filename = module.__file__
            except (AttributeError, ImportError):
                continue
            if filename is not None:
                filenames.append(filename)
        for filename in filenames:
            try:
                stat = os.stat(filename)
                if stat:
                    mtime = stat.st_mtime
                else:
                    mtime = 0
            except (OSError, IOError):
                continue
            if filename.endswith('.pyc') and os.path.exists(filename[:-1]):
                mtime = max(os.stat(filename[:-1]).st_mtime, mtime)
            if not filename in self.module_mtimes:
                self.module_mtimes[filename] = mtime
            elif self.module_mtimes[filename] < mtime:
                print("%s changed; reloading..." % filename)
                return False
        return True

    def watch_file(self, cls, filename):
        """Watch the named file for changes"""
        filename = os.path.abspath(filename)
        if self is None:
            for instance in cls.instances:
                instance.watch_file(filename)
            cls.global_extra_files.append(filename)
        else:
            self.extra_files.append(filename)

    watch_file = classinstancemethod(watch_file)

    def add_file_callback(self, cls, callback):
        """Add a callback -- a function that takes no parameters -- that will
        return a list of filenames to watch for changes."""
        if self is None:
            for instance in cls.instances:
                instance.add_file_callback(callback)
            cls.global_file_callbacks.append(callback)
        else:
            self.file_callbacks.append(callback)

    add_file_callback = classinstancemethod(add_file_callback)

watch_file = Monitor.watch_file
add_file_callback = Monitor.add_file_callback



# -*- encoding: utf-8 -*-
"""
Pyramid App
===========

"""
from pyramid.config import Configurator

from ginsfsm.globals import (
    set_global_app,
    get_global_app,
)

from banco.pyramid_app.pyramid_root import PyramidRoot


#-------------------------------------------------#
#       Paste entry point
#       Pyramid wsgi-app configuration
#-------------------------------------------------#
def pyramid_wsgi_app(global_config, **local_conf):
    """ Returns a WSGI application.
    """
    if 'gaplic' in local_conf:
        gaplic_name = local_conf.pop('gaplic')
    else:
        raise Exception('You must supply a gaplic name in ini file.')

    gaplic = get_global_app(gaplic_name)
    if not gaplic:
        raise Exception("gaplic '%s' supplied for wsgi is invalid" %
                        gaplic_name)

    config = Configurator(
        root_factory=get_root,
        settings=local_conf,
    )
    config.add_static_view(
        'static',
        'banco:pyramid_app/static',
        cache_max_age=3600
    )

    # Set up sockjs views
    config.include("ginsfsm.protocols.sockjs.server")

    # Set up views
    config.scan("banco.pyramid_app")

    # Now we have the gaplic and the config for make the root
    make_root(gaplic, config)

    return config.make_wsgi_app()


#----------------------------------------------------------------------#
#                           Pyramid root
#----------------------------------------------------------------------#
app_root = None


def make_root(gaplic, config):
    global app_root
    if app_root is None:
        app_root = gaplic.create_gobj(
            None,   # pyramid resources tree root MUST be unnamed.
            PyramidRoot,
            None    # and must have no parent.
        )
        set_global_app('banco_app', app_root)
    return app_root


def get_root(request):
    return app_root



# -*- coding: utf-8 -*-
"""
    Xhr implementation
"""
import logging

from pyramid.view import view_config
from pyramid.httpexceptions import HTTPNotFound, HTTPServerError

from ginsfsm.gobj import GObj
from ginsfsm.gconfig import GConfig
from ginsfsm.protocols.sockjs.server import proto
from ginsfsm.protocols.sockjs.server.basehandler import SessionHandler
from ginsfsm.protocols.sockjs.server.util import bytes_to_str

#----------------------------------------------------------------#
#                   GXhrSend GClass
#                   /xhr_send
#----------------------------------------------------------------#
GXHRSEND_GCONFIG = {
    'sockjs_server': [None, None, GConfig.FLAG_DIRECT_ATTR, None, ""],
}


class GXhrSend(GObj):
    """  GXhr GObj.
    """
    def __init__(self):
        GObj.__init__(self, {}, GXHRSEND_GCONFIG)

    def start_up(self):
        """ Initialization zone.
        """


#----------------------------------------------------------------#
#                   GXhrSend Views
#                   /xhr_send
#----------------------------------------------------------------#
@view_config(
    context=GXhrSend,
    name='',
    attr='options',
    request_method='OPTIONS',
)
@view_config(
    context=GXhrSend,
    name='',
    attr='post',
    request_method='POST',
)
class XhrSendHandler(SessionHandler):
    def post(self):
        response = self.response

        # Start response
        self.preflight()
        self.handle_session_cookie()
        self.disable_cache()

        session_id = self.sid = self.context.parent.re_matched_name
        session = self.context.sockjs_server.get_session(session_id)
        if session is None:
            return HTTPNotFound('Session not found')

        data = self.request.body_file.read()
        if not data:
            logging.error('ERROR xhr_send: Payload expected.')
            return HTTPServerError("Payload expected.")

        try:
            messages = proto.json_decode(bytes_to_str(data))  # ? v1.0.0
        except Exception:
            logging.exception(
                'ERROR xhr_send: Invalid json encoding',
            )
            return HTTPServerError("Broken JSON encoding.")

        try:
            session.on_messages(messages)
        except Exception:
            logging.exception(
                'ERROR xhr_send: on_message() failed',
            )
            return HTTPServerError('ERROR xhr_send: on_message() failed')

        self.set_status(204)
        self.response.content_type = 'text/plain'
        self.response.charset = 'UTF-8'
        return response


#----------------------------------------------------------------#
#                   GXhrPolling GClass
#                   /xhr
#----------------------------------------------------------------#
GXHRPOLLING_GCONFIG = {
    'sockjs_server': [None, None, GConfig.FLAG_DIRECT_ATTR, None, ""],
}


class GXhrPolling(GObj):
    """  GXhrPolling GObj.
    """
    def __init__(self):
        GObj.__init__(self, {}, GXHRPOLLING_GCONFIG)

    def start_up(self):
        """ Initialization zone.
        """


#----------------------------------------------------------------#
#                   GXhrPolling Views
#                   /xhr
#----------------------------------------------------------------#
@view_config(
    context=GXhrPolling,
    name='',
    attr='options',
    request_method='OPTIONS',
)
@view_config(
    context=GXhrPolling,
    name='',
    attr='post',
    request_method='POST',
)
class XhrPollingTransport(SessionHandler):
    """xhr-polling transport implementation"""
    name = 'xhr'

    def __init__(self, context, request):
        super(XhrPollingTransport, self).__init__(
            context,
            request,
            False
        )
        self.session = None
        self.active = True

    def post(self):
        response = self.response

        # Start response
        self.preflight()
        self.handle_session_cookie()
        self.disable_cache()

        session_id = self.sid = self.context.parent.re_matched_name

        # Get or create session without starting heartbeat
        session = self.context.sockjs_server.get_session(self.sid)
        if session is None:
            session = self.context.sockjs_server.create_session(session_id)
        if session is None:
            return HTTPServerError("ERROR creating session.")

        # Try to attach to the session
        if not session.set_handler(self, False):
            return response
        self.session = session
        # Verify if session is properly opened
        session.verify_state()

        # Might get already detached because connection was closed in on_open
        if not self.session:
            return response

        if not self.session.send_queue:
            self.session.start_heartbeat()
        else:
            self.session.flush()
        return response

    def send_pack(self, message, binary=False):
        if binary:
            raise Exception('binary not supported for XhrPollingTransport')

        if not self.active:
            return
        self.active = False

        try:
            self.response.content_type = 'application/javascript'
            self.response.charset = 'UTF-8'

            self.write(message + '\n')
            self.flush()
            self.send_complete()
        except Exception:
            # If connection dropped, make sure we close offending session
            # instead of propagating error all way up.
            logging.exception("ERROR XhrPollingTransport send_pack")
            if self.session:
                self.session.delayed_close()

    def send_complete(self):
        if self.session:  # detach session
            self.session.remove_handler(self)
            self.session = None
        self.active = True

    def session_closed(self):
        """Called by the session when it was closed"""
        if self.session:  # detach session
            self.session.remove_handler(self)
            self.session = None
        self.context.gaplic.add_callback(self.safe_finish)


#----------------------------------------------------------------#
#                   GXhrStreaming GClass
#                   /xhr_streaming
#----------------------------------------------------------------#
GXHRSTREAMING_GCONFIG = {
    'sockjs_server': [None, None, GConfig.FLAG_DIRECT_ATTR, None, ""],
}


class GXhrStreaming(GObj):
    """  GStreaming GObj.
    """
    def __init__(self):
        GObj.__init__(self, {}, GXHRSTREAMING_GCONFIG)

    def start_up(self):
        """ Initialization zone.
        """


#----------------------------------------------------------------#
#                   GXhrStreaming Views
#                   /xhr_streaming
#----------------------------------------------------------------#
@view_config(
    context=GXhrStreaming,
    name='',
    attr='options',
    request_method='OPTIONS',
)
@view_config(
    context=GXhrStreaming,
    name='',
    attr='post',
    request_method='POST',
)
class XhrStreamingTransport(SessionHandler):
    name = 'xhr_streaming'

    def __init__(self, context, request):
        super(XhrStreamingTransport, self).__init__(
            context,
            request,
            True  # Asynchronous response!
        )
        self.session = None
        self.active = True
        self.amount_limit = self.context.sockjs_server.config.response_limit

    def post(self):
        response = self.response

        session_id = self.sid = self.context.parent.re_matched_name

        # Start response
        self.preflight()
        self.handle_session_cookie()
        self.disable_cache()

        response.content_type = 'application/javascript'
        response.charset = 'UTF-8'
        response.write('h' * 2048 + '\n')

        # Get or create session without starting heartbeat
        session = self.context.sockjs_server.get_session(session_id)
        if session is None:
            session = self.context.sockjs_server.create_session(session_id)
        if session is None:
            # close the session in the next cycle.
            self.context.gaplic.add_callback(self.session_closed)
            return response  # how inform of the error? headers has been sent.

        # Try to attach to the session
        if not session.set_handler(self, False):
            # close the session in the next cycle.
            self.context.gaplic.add_callback(self.session_closed)
            return response  # how inform of the error? headers has been sent.
        self.session = session
        # Verify if session is properly opened
        session.verify_state()
        session.flush()

        return response

    def send_pack(self, message, binary=False):
        if binary:
            raise Exception('binary not supported for XhrStreamingTransport')

        #self.active = False  # TODO si chequeo active fallan los tests

        try:
            self.notify_sent(len(message))
            self.write(message + '\n')
            self.flush(callback=self.send_complete)
        except Exception:
            # If connection dropped, make sure we close offending session
            # instead of propagating error all way up.
            if self.session:  # detach session
                self.session.delayed_close()
                self.session.remove_handler(self)
                self.session = None
            logging.exception("ERROR XhrStreamingTransport send_pack")

    def send_complete(self):
        """
            Verify if connection should be closed based on amount of data
            that was sent.
        """
        self.active = True

        if self.should_finish():
            if self.session:  # detach session
                self.session.remove_handler(self)
                self.session = None
            self.context.gaplic.add_callback(self.safe_finish)
        else:
            if self.session:
                self.session.flush()

    def notify_sent(self, data_len):
        """
            Update amount of data sent
        """
        self.amount_limit -= data_len

    def should_finish(self):
        """
            Check if transport should close long running connection after
            sending X bytes to the client.

            `data_len`
                Amount of data that was sent
        """
        if self.amount_limit <= 0:
            return True

        return False

    def session_closed(self):
        """Called by the session when it was closed"""
        if self.session:  # detach session
            self.session.remove_handler(self)
            self.session = None
        self.context.gaplic.add_callback(self.safe_finish)



"""
GObj :class:`SimpleHttpServer`
==============================

Http serve sample.

The server echo the data.

It uses :class:`ginsfsm.protocols.http.server.c_http_server.GHttpServer`.

.. autoclass:: SimpleHttpServer
    :members:

"""

import logging
logging.basicConfig(level=logging.DEBUG)

from ginsfsm.gaplic import GAplic
from ginsfsm.smachine import SMachine
from ginsfsm.c_sock import GSock
from ginsfsm.gobj import GObj
from ginsfsm.protocols.http.server.c_http_server import GHttpServer
from ginsfsm.protocols.http.common.response import HttpSimpleOkResponse


def ac_channel_opened(self, event):
    print("======> channel opened")


def ac_channel_closed(self, event):
    print("======> channel closed")


def ac_request(self, event):
    print("======> channel request")
    channel = event.source[-1]
    request = event.request
    response = HttpSimpleOkResponse(
        request,
        'Caraculo\r\nCaraculo\r\nCaraculo\r\n'
    )
    self.send_event(channel, 'EV_HTTP_RESPONSE', response=response)


HTTP_ECHO_FSM = {
    'event_list': (
        'EV_HTTP_CHANNEL_OPENED: bottom input',
        'EV_HTTP_CHANNEL_CLOSED: bottom input',
        'EV_HTTP_REQUEST: bottom input',
    ),
    'state_list': ('ST_IDLE',),
    'machine': {
        'ST_IDLE':
        (
            ('EV_HTTP_CHANNEL_OPENED',  ac_channel_opened,  'ST_IDLE'),
            ('EV_HTTP_CHANNEL_CLOSED',  ac_channel_closed,  'ST_IDLE'),
            ('EV_HTTP_REQUEST',         ac_request,         'ST_IDLE'),
        ),
    }
}


HTTP_ECHO_GCONFIG = {  # type, default_value, flag, validate_function, desc
    'verbose': [int, 1, 0, None, "Increase output verbosity. Values [0,1,2]"],
}


class SimpleHttpServer(GObj):
    """  Server GObj.

    .. ginsfsm::
       :fsm: HTTP_ECHO_FSM
       :gconfig: HTTP_ECHO_GCONFIG

    *Input-Events:*


    *Output-Events:*


    """
    def __init__(self):
        GObj.__init__(self, HTTP_ECHO_FSM, HTTP_ECHO_GCONFIG)

    def start_up(self):
        self.server = self.create_gobj(
            None,
            GHttpServer,
            self,
            host='0.0.0.0',
            port=8000,
            subscriber=self,
            inactivity_timeout=10,
            responseless_timeout=2,
            maximum_simultaneous_requests=0,
        )

        if self.config.verbose > 0:
            settings = {
                'GObj.trace_mach': True,
                'GObj.logger': logging,
            }
            self.overwrite_parameters(-1, **settings)
            SMachine.global_trace_mach(True)
            GSock._global_trace_dump = True


#===============================================================
#                   Main
#===============================================================
if __name__ == "__main__":
    ga_srv = GAplic(name='Server', roles='')
    ga_srv.create_gobj(
        'server',
        SimpleHttpServer,
        ga_srv,
    )

    try:
        ga_srv.start()
    except (KeyboardInterrupt, SystemExit):
        print('Program stopped')



""" In the process domain there is no a only king.
There are applications (_global_apps),
there are threads (_global_threads),
and there are subprocesses (_global_subprocesses).

For now, the applications are only wsgi applications
to be used with wsgi-servers. Enough!!

Threads and subprocesses have their own king: his :term:`gaplic`.

And me, the main king, the main process of course.

"""
import atexit
import threading

_global_apps = {}
""" _global_apps is a dictionary with the wsgi applications
that are loaded from gaplic_factory when ini config file are processed.

They are defined in the ini config file, in ``wsgi`` item:

        [composite:main]
        wsgi = wsgi-application

        [app:wsgi-application]
        use = call:ginsfsm.examples.example6-wsgi-server:paste_app_factory

"""


def get_global_app(app_name):
    global _global_apps
    return _global_apps.get(app_name, None)


def set_global_app(app_name, callable):
    global _global_apps
    _global_apps[app_name] = callable


from collections import deque

_global_main = None
_global_threads = deque()
_global_subprocesses = deque()


def set_global_main_gaplic(gaplic):
    global _global_main
    _global_main = gaplic

    # save main gaplic in global apps, to could be retrieved by name
    if gaplic.name:
        set_global_app(gaplic.name, gaplic)


def get_global_main_gaplic():
    global _global_main
    return _global_main


def add_global_thread(worker):
    global _global_threads
    _global_threads.append(worker)


def get_global_threads():
    global _global_threads
    return _global_threads


def add_global_subprocess(worker):
    global _global_subprocesses
    _global_subprocesses.append(worker)


def get_global_subprocesses():
    global _global_subprocesses
    return _global_subprocesses


@atexit.register
def shutdown():
    """ Stop all gaplic, main, threads and subprocesses.
    """
    timeout = 5  # enough seconds?

    global_threads = get_global_threads()
    while 1:
        try:
            wkr = global_threads.pop()
        except IndexError:
            break
        wkr.gaplic.stop()
        wkr.join(timeout)

    global_subprocesses = get_global_subprocesses()
    while 1:
        try:
            wkr = global_subprocesses.pop()
        except IndexError:
            break
        wkr.gaplic.stop()
        wkr.join(timeout)

    global_main = get_global_main_gaplic()
    if global_main:
        global_main.stop()


def list_globals():
    """ List global_main, threads and subprocesses.
    """
    main_gaplic = get_global_main_gaplic()
    if main_gaplic:
        print("Global Main ==>")
        print("            --> %r, %r" % (main_gaplic.name, main_gaplic))

    global_threads = get_global_threads()
    for wkr in global_threads:
        print("Threads     ==>")
        print("            --> %r, %r" % (wkr.name, wkr))

    # TODO: will work???
    global_subprocesses = get_global_subprocesses()
    for wkr in global_subprocesses:
        print("Subprocess  ==>")
        print("            --> %r, %r" % (wkr.name, wkr))


def global_get_gobj(gobj_name, gaplic_name=None):
    """ Search a :term:`named-gobj` in global_main, threads and subprocesses.
    TODO: with subprocesses will work??
    """
    global_main_gaplic = get_global_main_gaplic()
    if global_main_gaplic:
        if not gaplic_name or global_main_gaplic.name == gaplic_name:
            gobj = global_main_gaplic.find_unique_gobj(gobj_name)
            if gobj:
                return gobj

    global_threads = get_global_threads()
    for wkr in global_threads:
        if not gaplic_name or wkr.gaplic.name == gaplic_name:
            gobj = wkr.gaplic.find_unique_gobj(gobj_name)
            if gobj:
                return gobj

    # TODO: will work???
    global_subprocesses = get_global_subprocesses()
    for wkr in global_subprocesses:
        if not gaplic_name or wkr.gaplic.name == gaplic_name:
            gobj = wkr.gaplic.find_unique_gobj(gobj_name)
            if gobj:
                return gobj
    return None


def global_get_gaplic(gaplic_name):
    """ Search a :term:`named-gobj` in global_main, threads and subprocesses.
    TODO: with subprocesses will work??
    """
    global_main_gaplic = get_global_main_gaplic()
    if global_main_gaplic:
        if global_main_gaplic.name == gaplic_name:
            return global_main_gaplic

    global_threads = get_global_threads()
    for wkr in global_threads:
        if wkr.gaplic.name == gaplic_name:
            return wkr.gaplic

    # TODO: will work???
    global_subprocesses = get_global_subprocesses()
    for wkr in global_subprocesses:
        if wkr.gaplic.name == gaplic_name:
            return wkr.gaplic

    return None


def get_gaplic_by_thread_ident(thread_ident=None):
    """ Return the gaplic by the thread ident.
        If :param:`thread_ident` is None,
        then search by the caller thread ident.
    """
    if thread_ident is None:
        thread_ident = threading.current_thread().ident
    global_main_gaplic = get_global_main_gaplic()
    if global_main_gaplic is None:
        print(
            "ERROR global main gaplic is NOT SET. "
            "Have you called set_global_main_gaplic()?."
        )
        raise Exception('ERROR global main gaplic not set.')
    if global_main_gaplic.thread_ident == thread_ident:
        return global_main_gaplic

    global_threads = get_global_threads()
    for wkr in global_threads:
        if wkr.gaplic.thread_ident == thread_ident:
            return wkr.gaplic

    # TODO: will work???
    global_subprocesses = get_global_subprocesses()
    for wkr in global_subprocesses:
        if wkr.gaplic.thread_ident == thread_ident:
            return wkr.gaplic
    return None



# -*- coding: utf-8 -*-
"""
    Sockjs Websocket transport implementation
"""
import logging

from pyramid.view import view_config

from ginsfsm.protocols.sockjs.server.proto import json_decode
from ginsfsm.gobj import GObj
from ginsfsm.gconfig import GConfig
from ginsfsm.protocols.sockjs.server.c_websocket import GWebSocket
from ginsfsm.protocols.sockjs.server.session import BaseSession
from ginsfsm.protocols.sockjs.server.session import ConnectionInfo
from ginsfsm.protocols.wsgi.webob.websocket_response import WebsocketResponse
from ginsfsm.deferred import Deferred
from ginsfsm.utils import hexdump
from ginsfsm.protocols.sockjs.server.util import bytes_to_str


#----------------------------------------------------------------#
#                   WebSocketHandler
#----------------------------------------------------------------#
class WebSocketHandler(object):
    """ Mixin to adapt GWebSocket and Sockjs session handler
    """

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.channel = channel = request.environ['ginsfsm.channel']
        self.gsock = channel.gsock

    def get_conn_info(self):
        """Return `ConnectionInfo` object from current transport"""
        return ConnectionInfo(
            self.request.remote_addr,  # remote_ip
            self.request.cookies,
            self.request.params,  # arguments
            self.request.headers,
            self.request.path
        )

    def _execute(self):
        #
        #   Accept the connection
        #
        self.ws_connection = self.context.gaplic.create_gobj(
            'websocket',
            GWebSocket,
            self.channel,     # child of parent, to be delete when channel was
            request=self.request,
            gsock=self.gsock,
        )
        self.ws_connection.delete_all_subscriptions()
        # da error al suscribir, no es un gobj, como hago para las callback?
        deferred_open = Deferred(0, self.open)
        deferred_message = Deferred(0, self.on_message)
        deferred_close = Deferred(0, self.on_close)
        self.ws_connection.subscribe_event('EV_ON_OPEN', deferred_open)
        self.ws_connection.subscribe_event('EV_ON_MESSAGE', deferred_message)
        self.ws_connection.subscribe_event('EV_ON_CLOSE', deferred_close)

        #
        #   This will execute ResponseInterrupt.
        #   Now Pyramid has nothing to do.
        #   All server/client dialog will be done by GWebSocket.
        #
        response = WebsocketResponse(self.context, self.request)
        return response

    def write_message(self, message, binary=False):
        """Sends the given message to the client of this Web Socket."""
        if self.ws_connection.ginsfsm_user_agent:
            self.ws_connection.send_jsonfied_message(message)
        else:
            self.ws_connection.write_message(message, binary)

    def close(self):
        self.gsock.mt_drop()

#----------------------------------------------------------------#
#                   GWebsocket GClass
#                   /*/*/websocket
#----------------------------------------------------------------#
GWEBSOCKET_GCONFIG = {
    'sockjs_server': [None, None, GConfig.FLAG_DIRECT_ATTR, None, ""],
}


class GWebsocket(GObj):
    """  GWebsocket GObj.
    """
    def __init__(self):
        GObj.__init__(self, {}, GWEBSOCKET_GCONFIG)

    def start_up(self):
        """ Initialization zone.
        """


#----------------------------------------------------------------#
#                   GWebsocket Views
#                   /*/*/websocket
#----------------------------------------------------------------#
@view_config(
    context=GWebsocket,
    name='',
    attr='execute',
)
class WebsocketTransport(WebSocketHandler):
    name = 'websocket'

    def __init__(self, context, request):
        super(WebsocketTransport, self).__init__(
            context,
            request,
        )
        self.session = None
        self.active = True

    def execute(self):
        self.sid = self.context.parent.re_matched_name
        response = self._execute()
        return response

    def open(self, event):
        # Handle session
        self.session = self.context.sockjs_server.create_session(
            self.sid,
            register=False
        )

        if not self.session.set_handler(self):
            self.close()
            return

        self.session.verify_state()

        if self.session:
            self.session.flush()

    def _detach(self):
        if self.session is not None:
            self.session.remove_handler(self)
            self.session = None

    def on_message(self, event):
        message = event.data
        # SockJS requires that empty messages should be ignored
        if not message or not self.session:
            return

        try:
            msg = json_decode(bytes_to_str(message))  # ? v1.0.0

        except Exception:
            logging.exception(
                'ERROR WebSocket json_decode, gobj %r, msg %s' %
                (self, hexdump('<==', message)))
            # Close running connection
            self.gsock.mt_drop()
            return

        try:
            if isinstance(msg, list):
                self.session.on_messages(msg)
            else:
                self.session.on_messages((msg,))
        except Exception:
            logging.exception(
                'ERROR WebSocket on_messages, gobj %r, msg %s' %
                (self, hexdump('<==', message)))
            # Close running connection
            self.gsock.mt_drop()

    def on_close(self, event):
        # Close session if websocket connection was closed
        if self.session is not None:
            # Detach before closing session
            session = self.session
            self._detach()
            session.close()

    def send_pack(self, message, binary=False):
        # Send message
        try:
            self.write_message(message, binary)
        except IOError:
            logging.exception(
                "ERROR WebsocketTransport send_pack")
            self.context.gaplic.add_callback(self.on_close)

    def session_closed(self):
        # If session was closed by the application, terminate websocket
        # connection as well.
        try:
            self.close()
        except IOError:
            pass
        finally:
            self._detach()

    def auto_decode(self):
        return False


#----------------------------------------------------------------#
#                   RawWebsocket Session
#----------------------------------------------------------------#
class RawSession(BaseSession):
    """ Raw session without any sockjs protocol encoding/decoding.
        Simply works as a proxy between `SockJSConnection` class
        and `RawWebSocketTransport`.
    """
    def send_message(self, msg, stats=True, binary=False):
        self.handler.send_pack(msg, binary)

    def on_message(self, msg):
        self.conn.on_message(msg)


#----------------------------------------------------------------#
#                   GRawWebsocket GClass
#                   /websocket
#----------------------------------------------------------------#
GRAWWEBSOCKET_GCONFIG = {
    'sockjs_server': [None, None, GConfig.FLAG_DIRECT_ATTR, None, ""],
}


class GRawWebsocket(GObj):
    """  GRawWebsocket GObj.
    """
    def __init__(self):
        GObj.__init__(self, {}, GRAWWEBSOCKET_GCONFIG)

    def start_up(self):
        """ Initialization zone.
        """


#----------------------------------------------------------------#
#                   GRawWebsocket Views
#                   /websocket
#----------------------------------------------------------------#
@view_config(
    context=GRawWebsocket,
    name='',
    attr='execute',
)
class RawWebsocketTransport(WebSocketHandler):
    name = 'rawwebsocket'

    def __init__(self, context, request):
        super(RawWebsocketTransport, self).__init__(
            context,
            request,
        )
        self.session = None
        self.active = True

    def execute(self):
        #self.sid = self.context.parent.re_matched_name
        response = self._execute()
        return response

    def open(self, event):
        # Create and attach to session
        self.session = RawSession(
            self.context.sockjs_server.get_connection_class(),
            self.context.sockjs_server
        )
        if not self.session.set_handler(self):
            self.close()
            return
        self.session.verify_state()

    def _detach(self):
        if self.session is not None:
            self.session.remove_handler(self)
            self.session = None

    def on_message(self, event):
        message = event.data
        # SockJS requires that empty messages should be ignored
        if not message or not self.session:
            return

        try:
            self.session.on_message(message)
        except Exception:
            logging.exception('ERROR RawWebSocket')

            # Close running connection
            # self._detach() ? v1.0.0
            self.gsock.mt_drop()

    def on_close(self, event):
        # Close session if websocket connection was closed
        if self.session is not None:
            # Detach before closing session
            session = self.session
            self._detach()
            session.close()

    def send_pack(self, message, binary=False):
        # Send message
        try:
            self.write_message(message, binary)
        except IOError:
            logging.exception(
                "ERROR RawWebsocketTransport send_pack")
            self.context.gaplic.add_callback(self.on_close)

    def session_closed(self):
        # If session was closed by the application, terminate websocket
        # connection as well.
        try:
            self.close()
        except IOError:
            pass
        finally:
            self._detach()



# -*- encoding: utf-8 -*-
"""
This module implements a simple Finite State Machine
(`FSM <http://en.wikipedia.org/wiki/Finite-state_machine>`_).

Simple Machine
--------------

We define the FSM by a :term:`simple-machine`, that it's running by
the :class:`SMachine` class.

A :term:`simple-machine` is a dictionary with three keys::

    FSM = {
        'event_list': (),
        'state_list': (),
        'machine': {}
    }

where:

* ``event_list``:
    item value is a tuple with all :term:`input-event` :term:`event-name`'s
    supported by the machine.
* ``state_list``:
    item value is a tuple with all :term:`state`'s of the machine.
* ``machine``:
    item value is another dictionary with the description of
    the :term:`machine`.

If something is wrong in the :term:`simple-machine` definition, a
:exc:`MachineError` exception it will be raised.

The ``machine`` item value is a another dictionary describing which
:term:`event-name`'s are supported by each :term:`state`,
the :term:`action` to be executed for each event, and what is
the :term:`next-state`.

The dictionary's keys are the state names.
The values are a tuple of tuples with three items:

    * :term:`event-name`.
    * :term:`action`.
    * :term:`next-state`.

where:
    **event-name**: event name, a name of the ``event_list`` tuple.

    **action**: function to be executed when the machine receives
    the :term:`event` through :meth:`SMachine.inject_event` method call.

        Their prototype is::

            def action(self, event):

        where ``event`` is the argument used in
        the :meth:`inject_event` method call.

    **next-state**: next state to set. Must be a name of ``state_list``
    or ``None``.

    .. note:: The next state is changed **before** executing
        the :term:`action` function.
        If you don't like this behavior, set to the `next-state`
        to ``None``. Then you can use the :meth:`SMachine.set_new_state`
        method to change the :term:`state` inside the :term:`action`
        function, otherwise, the machine will remain in the same
        state.

A :term:`simple-machine` example::

    def ac_task1(self, event):
        print 'TASK1'

    def ac_task2(self, event):
        print 'TASK2'

    def ac_task3(self, event):
        print 'TASK3'

    def ac_task4(self, event):
        print 'TASK4'

    SAMPLE_FSM = {
        'event_list': ('EV_EVENT1', 'EV_EVENT2'),
        'state_list': ('ST_STATE1', 'ST_STATE2'),
        'machine' = {
            'ST_STATE1':
            (
                ('EV_EVENT1',      ac_task1,       'ST_STATE2'),
                ('EV_EVENT2',      ac_task2,       'ST_STATE2'),
            ),
            'ST_STATE2':
            (
                ('EV_EVENT1',      ac_task3,       'ST_STATE1'),
                ('EV_EVENT2',      ac_task4,       'ST_STATE1'),
            ),
        }
    }

.. warning:: All event names and state names used in the :term:`machine`
    must be defined in :term:`event-list` and :term:`state-list`
    respectively, otherwise a :exc:`MachineError` will be raised.

.. note:: You can too have :term:`output-event` events.
   These events it's not necessary to be in :term:`event-list`.

.. note:: The list with :term:`output-event` events is not used
    by :class:`SMachine`. It is merely informative.

.. autoclass:: SMachine
    :members: inject_event,
        set_new_state, get_current_state,
        get_event_list, get_output_event_list

.. autoexception:: MachineError

.. autoexception:: EventError

.. autoexception:: StateError

.. autoexception:: EventNotAcceptedError

"""

import datetime
from ginsfsm.compat import (
    string_types,
    integer_types,
    iterkeys_,
    itervalues_,
)

EMPTY_FSM = {
    'event_list': (),
    'state_list': (),
    'machine': {}
}


class StateError(Exception):
    """ Raised when the state name doesn't match any name of
    the :term:`state-list` list."""


class EventError(Exception):
    """ Raised when the event name doesn't match any name of
    the :term:`event-list` list."""


class MachineError(Exception):
    """ Raised when something is wrong in the :term:`simple-machine`
    definition."""


class EventNotAcceptedError(Exception):
    """ Raised when the event name is good, but the event is not accepted
    in the current machine's state."""


def filter_event_attrs(event_list):
    filt_event_list = []
    for ev in event_list:
        ev_name = ev.split(':')[0]
        filt_event_list.append(ev_name)
    return filt_event_list


class SMachine(object):
    """ A class to run a :term:`simple-machine`.

    :param fsm: a :term:`simple-machine`, the dictionary describing the fsm.

    If the :term:`simple-machine` has errors a :exc:`MachineError` exception
    will be raised.
    """
    _inside = [0]    # to tab machine trace.
    _global_trace_mach = False

    def __init__(self, fsm):
        self.name = ''
        if not issubclass(fsm.__class__, dict):
            raise MachineError("SMachine(\"%s\") invalid TYPE in (%s,%s)" %
                               (repr(fsm), self.__class__.__name__, self.name))
        self._event_list = fsm.get('event_list', ())
        self._state_list = fsm.get('state_list', ())
        fsm = fsm.get('machine', {})
        self._states = []
        self._last_state = 0
        self._current_state = 1
        self.__trace_mach = False
        self.logger = None

        # check state names
        state_names = list(self._state_list)
        for st_name in iterkeys_(fsm):
            if st_name in state_names:
                state_names.remove(st_name)
            else:
                raise MachineError(
                    "machine state '%s' is NOT in state-list in (%s,%s)" %
                    (repr(st_name), self.__class__.__name__, self.name))
        if len(state_names):
            raise MachineError(
                "state-list OVERFILLED: %s in (%s,%s)" %
                (repr(state_names), self.__class__.__name__, self.name))

        # remove attributes from event_list and move attrs to _event_attr list
        event_list = []
        event_attrs = []
        for ev in self._event_list:
            ev_name = ev.split(':')[0]
            ev_name = ev_name.strip()
            ev_attrs = ev.split(':')[1:]
            event_list.append(ev_name)

            attrs_list = [attr.replace('.', ' ').replace(',', ' ').split()
                          for attr in ev_attrs]
            event_attrs.append(attrs_list)

        self._event_list = event_list
        self._event_attrs = event_attrs

        # build _output_set from event attributes
        output_set = set()
        for idx, ev_name in enumerate(self._event_list):
            attrs = self._event_attrs[idx]
            for attr in attrs:
                if 'output' in attr:
                    output_set.add(ev_name)
        self._output_set = output_set.copy()

        # check event names
        event_names = list(self._event_list)
        set_event_names = self._output_set.copy()  # start with output_set!
        for ev_ac in itervalues_(fsm):
            for ev_name, ac, nt in ev_ac:
                if ev_name in event_names:
                    set_event_names.add(ev_name)
                else:
                    raise MachineError(
                        "event name '%s' is NOT in event-list in (%s,%s)" %
                        (repr(ev_name), self.__class__.__name__, self.name))
        if len(event_names) != len(set_event_names):
            over = [x for x in event_names if x not in set_event_names]
            if len(over) == 0:
                for ev_name in set_event_names:
                    event_names.remove(ev_name)
                over = event_names
            raise MachineError(
                "event-list OVERFILLED: %s in (%s,%s)" %
                (repr(over), self.__class__.__name__, self.name))

        # check next state names and actions
        state_names = list(self._state_list)
        for ev_ac in itervalues_(fsm):
            for ev_name, ac, nt in ev_ac:
                if nt is not None and nt not in state_names:
                    raise MachineError(
                        "next statename '%s' is NOT in "
                        "state-list in (%s,%s)" %
                        (repr(nt), self.__class__.__name__, self.name))
                if ac is not None:
                    if not callable(ac):
                        raise MachineError(
                            "action '%s' is NOT callable in (%s,%s)" %
                            (repr(ac), self.__class__.__name__, self.name))

        # Build constant names (like C enum) for events: dict of name:id
        self._event_index = {'': 0}
        idx = 1
        for ev in self._event_list:
            self._event_index[ev] = idx
            idx = idx + 1

        # Build constant names (like C enum) for states: dict of name:id
        self._state_index = {'': 0}
        idx = 1
        for st in self._state_list:
            self._state_index[st] = idx
            idx = idx + 1

        #   Build list of states
        #   self._states is organized as:
        #
        #   [0]
        #       [0] [1] [2]... [n.events-1]
        #   [1]
        #       [0] [1] [2]... [n.events-1]
        #   [2]
        #       [0] [1] [2]... [n.events-1]
        #   ...
        #   [n.states-1]
        #       [0] [1] [2]... [n.events-1]
        #            |
        #            `--> [action, next_state]
        #
        #
        #   If a event is defined in a state, then,
        #   the element is a list([action,next_state]) instead of int.
        #
        #   This organization occupies more memory than necessary,
        #   but the execution is faster.
        #
        self._states = list(range(len(self._state_list) + 1))
        for st in self._state_list:
            st_idx = self._state_index[st]
            ev_action_list = fsm[st]
            self._states[st_idx] = list(range(len(self._event_list) + 1))

            for ev_act in ev_action_list:
                iev = self._event_index[ev_act[0]]
                # Get the action
                ac = ev_act[1]
                # Save the next state
                if ev_act[2] is not None:
                    next_state_id = self._state_index[ev_act[2]]
                else:
                    next_state_id = None
                # Save action/next-state
                self._states[st_idx][iev] = [ac, next_state_id]

    def set_new_state(self, new_state):
        """  Set a new state.
        Method to used inside actions, to force the change of state.

        :param new_state: new state to set.

        ``new_state`` must match some of the state names of the
        machine's :term:`state-list` or a :exc:`StateError` exception
        will be raised.
        """
        if not issubclass(new_state.__class__, (string_types)):
            raise StateError(
                "set_new_state(\"%s\") invalid TYPE in (%s,%s)" %
                (repr(new_state), self.__class__.__name__, self.name))

        state_id = self._state_index.get(new_state, 0)
        if state_id <= 0:
            raise StateError(
                "ERROR set_new_state %r UNKNOWN in (%s,%s)" %
                (new_state, self.__class__.__name__, self.name))
        self._set_new_state(state_id)

    def _set_new_state(self, state_id):
        if state_id is None or \
                not issubclass(state_id.__class__, integer_types) or \
                state_id <= 0 or state_id > len(self._state_list):
            raise StateError(
                "_set_new_state(\"%s\") state_id INVALID in (%s,%s)" %
                (repr(state_id), self.__class__.__name__, self.name))

        self._last_state = self._current_state
        self._current_state = state_id
        if self.trace_mach:
            if self._last_state != state_id:
                hora = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
                logger = self.logger
                logger and logger.info("%s%s - mach: (%s:%s), new_st: %s" % (
                    hora,
                    self._tab(),
                    self.__class__.__name__, self.name,
                    self._state_list[self._current_state - 1]))
        return True

    def get_current_state(self):
        """ Return the name of the current state.

        If there is no state it returns ``None``.
        """
        if self._current_state <= 0 or len(self._state_list) == 0:
            return None
        return self._state_list[self._current_state - 1]

    def inject_event(self, event):
        """  Inject an event into the FSM.

        :param event: a string event name or any object with an ``event_name``
                      attribute.

        .. warning:: use this method if you use the :class:`SMachine` class in
           isolation, otherwise use the methods to send events from
           the :class:`ginsfsm.gobj.GObj` class.

        Execute the associated :term:`action` to the ``event`` in the current
        state.

        The `event name` must match some of the :term:`event-name` of the
        machine's :term:`event-list` or a :exc:`EventError` exception
        will be raised.

        If the :term:`event-name` exists in the machine , but it's not accepted
        by the current state, then no exception is raised but the
        function **returns** :exc:`EventNotAcceptedError`.

            .. note:: The :meth:`inject_event` method doesn't
                **raise** :exc:`EventNotAcceptedError`
                because a :term:`machine` should run under any circumstances.
                In any way an action can raise exceptions.

        If the event is accepted by the machine, the returned value
        by :meth:`inject_event` method it will be the returned value by
        the executed :term:`action`.
        """

        result = None
        logger = self.logger
        trace_mach = self.trace_mach

        if issubclass(event.__class__, (string_types)):
            event_name = event
        else:
            if not hasattr(event, 'event_name'):
                raise EventError(
                    "inject_event(\"%s\") invalid TYPE in (%s,%s)"
                    % (self.__class__.__name__, self.name, repr(event)))
            event_name = event.event_name

        event_id = self._event_index.get(event_name, 0)
        if event_id <= 0:
            raise EventError("ERROR inject_event %r UNKNOWN in (%s,%s)" % (
                event_name, self.__class__.__name__, self.name))

        self._increase_inside()

        action = None
        if self._states:
            action = self._states[self._current_state][event_id]

        if not action or issubclass(action.__class__, integer_types):
            # Can be an int if there is no action defined to this event.
            # because of the use of range().

            hora = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            logger and logger.warn(
                "%s%s<> mach: (%s:%s), st: %s, ev: %s "
                "(NOT ACCEPTED, no match action)" % (
                    hora,
                    self._tab(),
                    self.__class__.__name__, self.name,
                    self._state_list[self._current_state - 1],
                    event.event_name
                )
            )
            self._decrease_inside()
            return EventNotAcceptedError

        if trace_mach:
            action_name = ''
            if action[0]:
                action_name = action[0].__name__
            hora = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            logger and logger.info(
                "%s%s-> mach: (%s:%s), st: %s, ev: (%s,%s), ac: %s()" % (
                    hora,
                    self._tab(),
                    self.__class__.__name__, self.name,
                    self._state_list[self._current_state - 1],
                    self._event_list[event_id - 1],
                    '' if trace_mach == 1 else repr(event.kw),
                    action_name)
            )

        if action[1] is not None:
            self._set_new_state(action[1])

        if action[0]:
            # Action found, execute
            result = action[0](self, event)

        if trace_mach:
            hora = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            logger and logger.info("%s%s<- mach: (%s:%s), st: %s, ret: %s" % (
                hora,
                self._tab(),
                self.__class__.__name__, self.name,
                self._state_list[self._current_state - 1],
                repr(result)))

        self._decrease_inside()
        return result

    def get_event_list(self):
        """ Return the list of event's names supported by the machine.
        """
        return list(self._event_list)

    def get_output_event_list(self):
        """ Return the list with the :term:`output-event`'s names.
        """
        return list(self._output_set)

    def _events_with_attrs(self, searched_attrs):  # pragma: no cover
        """ Used for automatic documentacion drawing.
        """
        events = set()
        for idx, ev_name in enumerate(self._event_list):
            attrs = self._event_attrs[idx]
            for attr in attrs:
                x = [att in attr for att in searched_attrs]
                if all(x):
                    events.add(ev_name)
        return list(events)

    def _event_states_with_attrs(self, searched_attrs):  # pragma: no cover
        """ Used for automatic documentacion drawing.
        """
        states = set()
        for idx, ev_name in enumerate(self._event_list):
            attrs = self._event_attrs[idx]
            for attr in attrs:
                x = [att in attr for att in searched_attrs]
                if all(x):
                    states_filter = \
                        [att for att in attr if att not in searched_attrs]
                    for st in states_filter:
                        states.add(st)
        return list(states)

    def get_top_output_event_states(self, event_name):  # pragma: no cover
        """ Used for automatic documentacion drawing.
            Get the states being used by output event.
            Return a list of states, empty if all states are used.
        """
        evs = ['output', 'top']
        states = self._event_states_with_attrs(evs)
        return states

    def get_bottom_output_event_states(self, event_name):  # pragma: no cover
        """ Used for automatic documentacion drawing.
            Get the states being used by output event.
            Return a list of states, empty if all states are used.
        """
        evs = ['output', 'bottom']
        states = self._event_states_with_attrs(evs)
        return states

    def get_top_input_events(self):  # pragma: no cover
        """ Used for automatic documentacion drawing.
        """
        evs = ['input', 'top']
        events = self._events_with_attrs(evs)
        return events

    def get_top_output_events(self):  # pragma: no cover
        """ Used for automatic documentacion drawing.
        """
        evs = ['output', 'top']
        events = self._events_with_attrs(evs)
        return events

    def get_bottom_input_events(self):  # pragma: no cover
        """ Used for automatic documentacion drawing.
        """
        evs = ['input', 'bottom']
        events = self._events_with_attrs(evs)
        return events

    def get_bottom_output_events(self):  # pragma: no cover
        """ Used for automatic documentacion drawing.
        """
        evs = ['output', 'bottom']
        events = self._events_with_attrs(evs)
        return events

    @staticmethod
    def global_trace_mach(value):
        value = True if value else False
        SMachine._global_trace_mach = value

    @property
    def trace_mach(self):
        return self.__trace_mach or self._global_trace_mach

    @trace_mach.setter
    def trace_mach(self, value):
        """  Enable/Disable machine trace.
        If value is higher than 1, the data event is traced.
        """
        logger = self.logger
        old_value = self.__trace_mach
        new_value = value
        hora = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        if not old_value and new_value:
            self.__trace_mach = new_value
            logger and logger.info("%s%s* trace_mach ON: (%s:%s)" % (
                hora,
                self._tab(),
                self.__class__.__name__, self.name))
        elif old_value and not new_value:
            self.__trace_mach = False
            logger and logger.info("%s%s* trace_mach OFF: (%s:%s)" % (
                hora,
                self._tab(),
                self.__class__.__name__, self.name))

    def _increase_inside(self):
        """ TO BE OVERRIDEN by gaplic
        """
        self._inside[0] += 1

    def _decrease_inside(self):
        """ TO BE OVERRIDEN by gaplic
        """
        self._inside[0] -= 1

    def _tab(self):
        """ Indent, return spaces multiple of depth level gobj.
        With this, we can see the trace messages indenting according
        to depth level.
        TO BE OVERRIDEN by gaplic.
        """
        if self._inside[0] <= 0:
            spaces = 1
        else:
            spaces = self._inside[0] * 2
        pad = ' ' * spaces
        return pad



# -*- encoding: utf-8 -*-
"""
applic10
=======

Applic 10

"""
from pyramid.response import Response
from pyramid.view import view_config
from pyramid.security import (
    Allow,
    Authenticated,
    Everyone,
    DENY_ALL,
)

from ginsfsm.gobj import GObj
from ginsfsm.c_timer import GTimer


#----------------------------------------------------------------------#
#                           Applic1
#----------------------------------------------------------------------#

#-------------------------------------------#
#               Actions
#-------------------------------------------#
def ac_timeout(self, event):
    self.config.counter += 1
    self.send_event(self.timer, 'EV_SET_TIMER', seconds=self.config.seconds)


PYRAMIDROOT_FSM = {
    'event_list': (
        'EV_SET_TIMER: bottom output',
        'EV_TIMEOUT: bottom input',
    ),
    'state_list': (
        'ST_IDLE',
    ),
    'machine': {
        'ST_IDLE':
        (
            ('EV_TIMEOUT', ac_timeout, 'ST_IDLE'),
        ),
    }
}

PYRAMIDROOT_GCONFIG = {
    'pyramid_router_url': [
        str, '', 0, None,
        'pyramid url to start the router'
    ],
    'seconds': [int, 60, 0, None, "Timer to increment the counter."],
    'counter': [int, 0, 0, None, "Counter to display."],
}


class PyramidRoot(GObj):
    """  Root resource (/)

    .. ginsfsm::
       :fsm: PYRAMIDROOT_FSM
       :gconfig: PYRAMIDROOT_GCONFIG

    *Input-Events:*
        * :attr:`'EV_TIMEOUT'`: Timer over.

    """
    __acl__ = [
        (Allow, Everyone, 'view'),
        (DENY_ALL),
    ]

    def __init__(self):
        GObj.__init__(self, PYRAMIDROOT_FSM, PYRAMIDROOT_GCONFIG)

    def start_up(self):
        """ Initialization zone.
        """
        if self.config.pyramid_router_url:
            self.gaplic.start_up_router(
                pyramid_router_url=self.config.pyramid_router_url,
                pyramid_root=self
            )

        self.timer = self.create_gobj(
            None,       # unnamed gobj
            GTimer,     # gclass
            self        # parent
        )
        self.send_event(self.timer, 'EV_SET_TIMER', seconds=self.config.seconds)


#-------------------------------------------#
#               Views
#-------------------------------------------#
@view_config(
    context=PyramidRoot,
    permission='view',
    renderer='templates/mytemplate.pt')
def default_view(self, request):
    return {'project': 'banco'}



# -*- encoding: utf-8 -*-
"""
Simple Sockjs server
====================

It uses :class:`ginsfsm.protocols.wsgi.server.c_wsgi_server.GWsgiServer`.

You can run this file with ``gserve test_sockjs_server.ini``

.. autofunction:: main

"""

import logging
logging.basicConfig(level=logging.WARN)

from ginsfsm.gaplic import GAplic
from ginsfsm.globals import (
    set_global_app,
    set_global_main_gaplic,
)

from ginsfsm.protocols.wsgi.server.c_wsgi_server import GWsgiServer


#===============================================================
#       Paste app factory.
#       To run with gserve yyy.ini
#===============================================================
def paste_app_factory(global_config, **local_conf):
    from ginsfsm.examples.sockjs.test_sockjs_apps import pyramid_application
    return pyramid_application(global_config, **local_conf)


#===============================================================
#                   Main
#===============================================================
def main(global_config, **local_conf):
    """ Entry point to run with gserve (PasteDeploy)
    """
    if 'application' in local_conf:
        application = local_conf.pop('application')
    else:
        raise Exception('You must supply an wsgi application.')
    ga = GAplic(name='ga', roles='', **local_conf)
    set_global_main_gaplic(ga)
    ga.create_gobj(
        'test-sockjs',
        GWsgiServer,
        ga,
        application=application,
    )
    return ga


if __name__ == "__main__":
    """ You can run directly this file, without gserve.
    """
    import logging
    logging.basicConfig(level=logging.WARN)
    # simulate running from ini file
    local_conf = {
        'GObj.trace_mach': False,
        'GObj.logger': logging,
        'GSock.trace_dump': False,
        'GWsgiServer.host': '0.0.0.0',
        'GWsgiServer.port': 8080,
        'application': 'wsgi-application',
    }

    ga = main({}, **local_conf)

    applic_settings = {
        'pyramid.reload_templates': True,
        'pyramid.debug_authorization': False,
        'pyramid.debug_notfound': False,
        'pyramid.debug_routematch': False,
        'pyramid.debug_templates': True,
        'pyramid.default_locale_name': 'en',
        'gaplic': 'ga'
    }

    set_global_app(
        'wsgi-application',
        paste_app_factory({}, **applic_settings)
    )

    try:
        ga.start()
    except (KeyboardInterrupt, SystemExit):
        print('Program stopped')



# -*- encoding: utf-8 -*-
"""
    WebsocketResponse -  an asynchronous Websocket WebOb Response over ginsfsm.
"""
from ginsfsm.protocols.http.server.c_http_clisrv import ResponseInterrupt

try:
    from pyramid.response import Response
except ImportError:  # pragma: no cover
    try:
        from webob import Response
    except ImportError:  # pragma: no cover
        raise Exception('async_response is depending of Pyramid or WebOb')


class WebsocketResponse(Response):
    def __init__(self, context, request):
        super(WebsocketResponse, self).__init__()
        self.context = context
        self.request = request

    def __call__(self, environ, start_response):
        "Override WSGI call"
        raise ResponseInterrupt()

